#include "globmat.h"
#include "genfile.h"
#include "global.h"
#include "elemhead.h"
#include "loadcase.h"
#include "dloadcase.h"
#include "dloadpd.h"
#include "node.h"
#include "element.h"
#include "intpoints.h"
#include "problem.h"
#include "mechmat.h"
#include "creepb.h"
#include "creepbs.h"
#include "mechcrsec.h"
#include "elemswitch.h"



/**
  Function assembles stiffness %matrix.
  
  Parameters: 
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by JK,
*/
void stiffness_matrix (long lcid)
{
  long i,j,nl,ndofe,ndofn,nmult,nid,*cn,*cna;
  matrix lm,lmt;
  
  if (Smat==NULL)
    Smat = new gmatrix;
  Smat->setval (Mp->ssle);
  Smat->initiate (Gtm,Ndofm,Mp->tstorsm,Mespr,Out);
  
  //  the following condition is used only for problem type which
  //  deals with load balancing method
  //  the code stops here because it needs to compute number of matrix
  //  entries in the system matrix
  if (Mp->lbtype==1)
    abort ();
  
  
  /*
    FILE *ma;
    ma=fopen ("matice.txt","w");
  */
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      //  number of DOFs on element
      ndofe = Mt->give_ndofe (i);
      
      allocm (ndofe,ndofe,lm);
      //  computation of stiffness matrix of one element
      stiffmat (lcid,i,lm);
      
      
      //fprintf (ma,"\n%ld\n",i+1);
      //for (long j=0;j<lm.n;j++){
      //for (long k=0;k<lm.n;k++){
      //fprintf (ma," %le",lm.a[j*lm.n+k]);
      //}
      //fprintf (ma,"\n");
      //}

      //if (i==3000000){
      //fclose (ma);
      //ma=fopen ("matice2.txt","w");
      //}
      
      cn = new long [ndofe];
      Mt->give_code_numbers (i,cn);
      
      //  localization of matrix of one element to matrix of problem
      Smat->localize (lm,cn,i);
      
      destrm (lm);
      delete [] cn;
    }
  }
  
  /*
    fclose (ma);
    
    ma=fopen ("sit.txt","w");
    fprintf (ma,"%ld\n",Gtm->nn);
    for (i=0;i<Gtm->nn;i++){
    fprintf (ma,"%ld  %lf %lf %lf  %ld %ld %ld\n",i+1,Gtm->gnodes[i].x,Gtm->gnodes[i].y,Gtm->gnodes[i].z,Gtm->gnodes[i].cn[0],Gtm->gnodes[i].cn[1],Gtm->gnodes[i].cn[2]);
    }
    fprintf (ma,"\n\n%ld\n",Gtm->ne);
    for (i=0;i<Gtm->ne;i++){
    fprintf (ma,"%ld    %ld %ld %ld %ld\n",i+1,Gtm->gelements[i].nodes[0],Gtm->gelements[i].nodes[1],Gtm->gelements[i].nodes[2],Gtm->gelements[i].nodes[3]);
    //fprintf (ma,"%ld    %ld %ld %ld %ld %ld %ld %ld %ld\n",
    //i+1,Gtm->gelements[i].nodes[0],Gtm->gelements[i].nodes[1],Gtm->gelements[i].nodes[2],
    //Gtm->gelements[i].nodes[3],Gtm->gelements[i].nodes[4],Gtm->gelements[i].nodes[5],
    //Gtm->gelements[i].nodes[6],Gtm->gelements[i].nodes[7]);
    }
    fclose (ma);
  */
  
  
  
  /*
    DSS::SparseMatrixF sm((unsigned long)Ndofm,Smat->scr->a,(unsigned long*)Smat->scr->ci,(unsigned long*)Smat->scr->adr);
    FILE *out;
    out = fopen ("matice.sm","wb");
    sm.SaveMatrix(out);
  */
  
  if (Mp->tprob==layered_linear_statics){
    for (i=0;i<Mt->nln;i++){
      nl=Gtm->lgnodes[i].nl;
      ndofn=Gtm->lgnodes[i].ndofn;
      nmult=Gtm->lgnodes[i].nmult;
      //allocm (nmult,ndofn*2,lm);
      allocm (ndofn*2,nmult,lm);
      allocm (nmult,ndofn*2,lmt);
      cn = new long [ndofn*2];
      cna = new long [nmult];
      
      for (j=1;j<nl;j++){
	constr_matrix (i,j,lm);
	
	nid=Gtm->lgnodes[i].nodes[j-1];
	Mt->give_node_code_numbers (nid,cn);
	nid=Gtm->lgnodes[i].nodes[j];
	Mt->give_node_code_numbers (nid,cn+ndofn);
	Mt->give_mult_code_numbers (i,j,cna);
	
	tranm (lm,lmt);
	
	Smat->glocalize (lmt,cna,cn);
	Smat->glocalize (lm,cn,cna);
	
	
      }
      destrm (lm);
      destrm (lmt);
      delete [] cn;
      delete [] cna;
    }
  }
  
  Smat->prepmat (0.0,Mespr);
  
  //Smat->printdiag(Out);
  //Smat->printmat(Out);
}



/**
  Function assembles mass %matrix.
   
  Parameters:
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by JK,
*/
void mass_matrix (long lcid)
{
  long i,ndofe,*cn;
  matrix lm;
  
  if (Mmat==NULL)  Mmat = new gmatrix;
  //Mmat->ts=Mp->tstormm;
  Mmat->setval (Mp->ssle);
  Mmat->initiate (Gtm,Ndofm,Mp->tstormm,Mespr,Out);
  //Mmat->setval (Mp->ssle.tlinsol,Mp->ssle.prec.pt,Mp->ssle.ni,Mp->ssle.err);
  
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      //  number of DOFs on the i-th element
      ndofe = Mt->give_ndofe (i);
      //  allocation of element mass matrix
      allocm (ndofe,ndofe,lm);
      //  computation of mass matrix of one element
      massmat (lcid,i,lm);
      
      cn = new long [ndofe];
      Mt->give_code_numbers (i,cn);
      
      //  localization of matrix of element to matrix of problem
      Mmat->localize (lm,cn,i);
      
      destrm (lm);
      delete [] cn;
    }
  }
  
  /*
  for (i=0;i<Mt->nn;i++){
    cr = Mt->nodes[i].crst;
    if (cr==nocrosssection){
      continue;
    }
    else{
      ndofn = Mt->give_ndofn (i);
      for (j=0;j<ndofn;j++){
	k = Mt->give_dof (i,j);
	if (k>0){
	  idcs = Mt->nodes[i].idcs;
	  m=Mc->give_weight (cr,idcs);
	  Mmat->add_entry (m,k-1,k-1);
	}
      }
    }
  }
  */

  Mmat->prepmat (0.0,Mespr);
}



/**
  Function assembles initial stiffness %matrix.

  It is also called geometric stiffness %matrix
  (in R.W. Clough and J. Penzien: Dynamics of structures)

  Parameters:   
  @param lcid - load case id
   
  @return The function does not return anything.

  Created by JK,
*/
void initial_stiffness_matrix (long lcid)
{
  if (Ismat==NULL)  Ismat = new gmatrix;
  //Ismat->ts=Mp->tstorsm;
  Ismat->setval (Mp->ssle);
  Ismat->initiate (Gtm,Ndofm,Mp->tstorsm,Mespr,Out);
  //Ismat->setval (Mp->ssle.tlinsol,Mp->ssle.prec.pt,Mp->ssle.ni,Mp->ssle.err);
  
  
  long i,ndofe,*cn;
  matrix lm;
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){

      ndofe = Mt->give_ndofe (i);
      
      allocm (ndofe,ndofe,lm);
      
      //  computation of initial stiffness matrix of one element
      initstiffmat (lcid,i,lm);
      
      cn = new long [ndofe];
      Mt->give_code_numbers (i,cn);
      
      //  localization of matrix of one element to matrix of problem
      Ismat->localize (lm,cn,i);
      
      destrm (lm);
      delete [] cn;
    }
  }
  
  Ismat->prepmat (0.0,Mespr);
}



/**
  The function assembles damping %matrix.
   
  @return The function does not return anything.

  Created by JK, 10.8.2005
*/
void damping_matrix ()
{
  if (Dmat==NULL)  Dmat = new gmatrix;
  //Dmat->ts=Mp->tstorsm;
  Dmat->setval (Mp->ssle);
  Dmat->initiate (Gtm,Ndofm,Mp->tstorsm,Mespr,Out);
  //Dmat->setval (Mp->ssle.tlinsol,Mp->ssle.prec.pt,Mp->ssle.ni,Mp->ssle.err);
  
  switch (Mp->damp){
  case nodamp:{
    break;
  }
  case massdamp:{
    Dmat->addgm (Mp->dmass,*Mmat);
    break;
  }
  case stiffdamp:{
    Dmat->addgm (Mp->dstiff,*Smat);
    break;
  }
  case rayleighdamp:{
    Dmat->addgm (Mp->dmass,*Mmat);
    Dmat->addgm (Mp->dstiff,*Smat);
    break;
  }
  default:{
    print_err("unknown type of damping is required",__FILE__,__LINE__,__func__);
  }
  }
  
  Dmat->prepmat (0.0,Mespr);
}



/**
  The function computes %vector of internal forces.
   
  @param lcid - load case id
  @param intfor - %vetcor of internal forces (output)
   
  @return The function returns %vector of internal forces
  in the parameter intfor.

  Created by JK, 28.7.2001
*/
void internal_forces (long lcid,double *intfor)
{
  if (Mp->matmodel==local){
    loc_internal_forces (lcid,intfor);
  }
  if (Mp->matmodel==nonlocal){
    nonloc_internal_forces (lcid,intfor);
  }
  
  //  contribution to internal forces by Lagrange multipliers
  //  it is used in problems with floating subdomains solved by the FETI method
  //  JK, 22.6.2006
  if (Mp->tprob == lin_floating_subdomain || Mp->tprob == nonlin_floating_subdomain){
    lagrmultcontr_intforces (lcid,intfor);
  }

  
  //  indicator of strain computation
  //  it means, strains at integration points have been computed
  Mp->strainstate=1;
  //  indicator of stress computation
  //  it means, stresses at integration points have been computed
  Mp->stressstate=1;
  //  indicator of computation of other array
  //  it means, stresses at integration points have been computed
  Mp->otherstate=1;
}



/**
  The function computes %vector of internal forces,
  local material models are used.

  @param lcid - load case id
  @param intfor - %vector of internal forces (output)
   
  @return The function returns %vector of internal forces computed localy
  in the parameter intfor.

  Created by JK, TKo,  28.7.2001
*/
void loc_internal_forces (long lcid,double *intfor)
{
  long i,ne,ndofe;
  ivector cn;
  vector ifor;
  
  //  function computes strains at integration points
  compute_ipstrains (lcid);
    
  //  number of elements
  ne=Mt->ne;
  
  //  cleaning of the array intfor
  nullv (intfor,Ndofm);
  
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      ndofe=Mt->give_ndofe (i);
      allocv (ndofe,ifor);
      allocv (ndofe,cn);
      
      elem_internal_forces (i,lcid,ifor);
      
      Mt->give_code_numbers (i,cn.a);

      locglob (intfor,ifor.a,cn.a,ndofe);
      destrv (ifor);
      destrv (cn);
    }
  }
  
}



/**
  The function computes %vector of internal forces,
  nonlocal material models are used.

  @param lcid - load case id
  @param intfor - %vector of internal forces (output)

  @return The function returns %vector of internal forces computed nonlocaly
  in the parameter intfor.

  Created by JK,TKo, 28.7.2001
*/
void nonloc_internal_forces (long lcid,double *intfor)
{
  long i,ne,ndofe;
  ivector cn;
  vector ifor;

  //  function computes strains at integration points
  compute_ipstrains (lcid);

  ne=Mt->ne;
  nullv (intfor,Ndofm);
  Mp->nonlocphase=1;

  for (i = 0; i < ne; i++)
  {
    if (Gtm->leso[i]==1)
     elem_local_values (i, lcid);
  }
  
  for (i = 0; i < Mm->tnip; i++)
    Mm->nonlocaverage(i);
  Mp->nonlocphase=2;
  
  for (i = 0; i < ne; i++)
  {
    if (Gtm->leso[i]==1)
    {
      ndofe=Mt->give_ndofe (i);
      allocv (ndofe,ifor);
      allocv (ndofe,cn);
      elem_nonloc_internal_forces(i, lcid, ifor);
      Mt->give_code_numbers (i,cn.a);
      locglob (intfor,ifor.a,cn.a,ndofe);
      destrv (ifor);
      destrv (cn);
    }
  }
}



/**
  The function computes %vector of increments of internal forces.
   
  @param lcid - load case id
  @param intfor - %vector of increments of internal forces (output)
   
  @return The function returns %vector of increments of internal forces 
  in the parameter intfor.

  Created by JK, TKo, 29.4.2008
*/
void incr_internal_forces (long lcid,double *intfor)
{
  long i,ne,ndofe;
  ivector cn;
  vector ifor;
  
  //  function computes strains at integration points
  compute_ipstrains (lcid);
    
  //  number of elements
  ne=Mt->ne;
  
  //  cleaning of the array intfor
  nullv (intfor,Ndofm);
  
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      ndofe=Mt->give_ndofe (i);
      allocv (ndofe,ifor);
      allocv (ndofe,cn);
      
      elem_incr_internal_forces (i,lcid,ifor);
      
      Mt->give_code_numbers (i,cn.a);

      locglob (intfor,ifor.a,cn.a,ndofe);
      destrv (ifor);
      destrv (cn);
    }
  }
}



/**
  Function adds contributions to internal forces caused by
  Lagrange multipliers. Function is used in problems with floating subdomains
  solved by the FETI method.
   
  @param lcid - load case id
  @param intfor - array of internal forces (input/output)
   
  @return The function adds contributions to the parameter intfor.

  Created by JK, 22.6.2006
*/
void lagrmultcontr_intforces (long lcid,double *intfor)
{
  long i;
  double *av,*av1,*av2,*cv;
  
  av = new double [Ndofm];
  av1 = new double [Ndofm];
  av2 = new double [Ndofm];
  cv = new double [Fsd->nlm];
  
  //  E^T lamba
  nullv (av,Ndofm);
  nullv (av1,Ndofm);
  nullv (av2,Ndofm);
  //Gtm->flsub.coarse_local (Fsd->tw,av);
  Gtm->flsub.coarse_local (Fsd->muo,av1);
  Gtm->flsub.coarse_local (Fsd->ddmu,av2);
  
  for (i=0;i<Ndofm;i++){
    av[i]=av1[i]+av2[i];
  }

  //Gtm->flsub.local_coarse (cv,Lsrs->lhs);
  
  /*
  long i;
  for (i=0;i<Ndofm;i++){
    fprintf (Out,"\n lagr mult %le",av[i]);
  }
  for (i=0;i<Fsd->nlm;i++){
    fprintf (Out,"\n coarse vector %le",cv[i]);
    

    if (cv[i]>10.0){
      if (av[i]<0.0){
	av[i]=-10.0;
	Fsd->tw[i]=-10.0;
      }
      if (av[i]>0.0){
	av[i]=10.0;
	Fsd->tw[i]=10.0;
      }
      Fsd->compli[i]=2.0;
    }
    
  }
  */

  addv (intfor,av,Ndofm);
  
  delete [] av;
  delete [] cv;
}



/**
  The function computes %vector of nodal forces
  caused by eigenstrains. Eigenstrains may contain 
  either eigenstrains or temperature strains.
   
  @param lcid - load case id
  @param nodfor - %vector of nodal forces (output)
   
  @return The fucntion returns nodal forces in the parameter nodfor.

  Created by JK, 28.7.2001
*/
void nodal_eigstrain_forces (long lcid,double *nodfor)
{
  long i,ne,ndofe;
  elemtype te;
  ivector cn;
  vector nfor;
  
  //  array nodfor must not be cleaned
  //  it contains contributions from previous calculations
  //  this function should only contribute to this array
  
  //  number of elements
  ne=Mt->ne;
  
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      ndofe=Mt->give_ndofe (i);
      allocv (ndofe,nfor);
      allocv (ndofe,cn);
      
      //  computes contribution from one element
      elem_eigstrain_forces (lcid,i,nfor);
      
      Mt->give_code_numbers (i,cn.a);
      locglob (nodfor,nfor.a,cn.a,ndofe);
      destrv (nfor);
      destrv (cn);
    }
  }
}



/**
  The function extracts displacements on one element.
   
  @param lcid - number of load case
  @param eid - element id
  @param r - allocated array for displacement, it is output parameter
  @param cn - array containing code numbers
  @param ndofe - number of DOFs on actual element
   
  @return The function returns nodal displacements at the given element in the array r.

  Created by JK, 9.7.2001
*/
void eldispl (long lcid,long eid,double *r,long *cn,long ndofe)
{
  long i,j,k,ii;
  
  switch (Mp->tprob){
  case forced_dynamics:
  case mech_timedependent_prob:{
    //  lcid must be equal to zero
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   r[i]=Mb->dlc[lcid].get_pd(Mp->time, ii);
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
    break;
  }

  case mat_nonlinear_statics:{
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   r[i]=Mb->lc[lcid].pd[0-ii-1] * Mp->lambda + Mb->lc[lcid+1].pd[0-ii-1];
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
    break;
  }
    
  case growing_mech_structure:{
    //  lcid must be equal to zero
    long nne = Mt->give_nne(eid);
    long ndofn, acn;
    ivector nodes(nne);
    Mt->give_elemnodes(eid,nodes);
    k=0;
    acn = 0;
    for (i=0; i<nne; i++)
    {
      ndofn = Mt->give_ndofn(nodes[i]);
      for (j=0; j<ndofn; j++)
      {
        ii=cn[acn+j];
        if (ii<0)   r[acn+j]=Mb->dlc[lcid].get_pd(Mp->time, ii) + Mt->nodes[nodes[i]].nodval[j];
        if (ii==0)  r[acn+j]=0.0;
        if (ii>0)   r[acn+j]=Lsrs->lhs[lcid*Ndofm+ii-1];
      }
      acn += ndofn;
    }     
    //  subtraction of initial displacements
    Mt->elements[eid].subtrinitdispl (r,ndofe);
    break;
  }
    
  default:{
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   r[i]=Mb->lc[lcid].pd[0-ii-1];
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
  }
  }
}



/**
  The function extracts prescribed displacements on one element.
   
  @param lcid - number of load case
  @param r - array of prescribed displacements, it is output parameter
  @param cn - array containing code numbers
  @param ndofe - number of DOFs on actual element
   
  @return The function returns extracted prescribed displacements in the array r.

  Created by JK, 9.7.2001
*/
void elprdispl (long lcid,long eid,double *r,long *cn,long ndofe)
{
  long i,ii;
  
  switch (Mp->tprob){
  case forced_dynamics:
  case mech_timedependent_prob:{
    //  lcid must be equal to zero
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   
        r[i]=Mb->dlc[lcid].get_pd(Mp->time, ii);
    }
    break;
  }

  case mat_nonlinear_statics:{
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   
        r[i]=Mb->lc[lcid].pd[0-ii-1] * Mp->lambda + Mb->lc[lcid+1].pd[0-ii-1];
    }
    break;
  }
    
  case growing_mech_structure:{
    //  lcid must be equal to zero
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   
        r[i]=Mb->dlc[lcid].get_pd(Mp->time, ii);
    }    
    //  subtraction of initial displacements
    Mt->elements[eid].subtrinitdispl (r,ndofe);
    break;
  }
    
  default:{
    for (i=0;i<ndofe;i++){
      ii=cn[i];
      if (ii<0)   
        r[i]=Mb->lc[lcid].pd[0-ii-1];
    }
  }
  }
}



/**
  The function extracts displacements on one node.

  @param lcid  - number of load case
  @param r     - allocated array for displacement, it is output parameter
  @param nid   - node number (node id)

  @return The function returns nodal displacements at the given node in the array r.

  Created by JK 9.7.2001
  Modified by Tomas Koudelka
*/
void noddispl (long lcid,double *r, long nid)
{
  long i,ii;
  
  switch (Mp->tprob){
  case forced_dynamics:
  case mech_timedependent_prob:
  {
    for (i=0;i<Mt->give_ndofn (nid);i++)
    {
      ii=Mt->give_dof(nid,i);
      if (ii<0)   r[i]=Mb->dlc[lcid].get_pd(Mp->time, ii);
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
    break;
  }
  case growing_mech_structure:
    for (i=0;i<Mt->give_ndofn (nid);i++)
    {
      ii=Mt->give_dof(nid,i);
      if (ii<0)   r[i]=Mb->dlc[lcid].get_pd(Mp->time, ii) + Mt->nodes[nid].nodval[i];
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
    break;
  case mat_nonlinear_statics:{
    for (i=0;i<Mt->give_ndofn (nid);i++){
      ii=Mt->give_dof(nid,i);
      if (ii<0)   r[i]=Mb->lc[lcid].pd[0-ii-1]*Mp->lambda + Mb->lc[lcid+1].pd[0-ii-1];
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
    break;
  }
  default:{
    for (i=0;i<Mt->give_ndofn (nid);i++){
      ii=Mt->give_dof(nid,i);
      if (ii<0)   r[i]=Mb->lc[lcid].pd[0-ii-1];
      if (ii==0)  r[i]=0.0;
      if (ii>0)   r[i]=Lsrs->lhs[lcid*Ndofm+ii-1];
    }
  }
  }
}



/**
  The function extracts displacements on one node for given problem.
   
  @param lcid - number of load case
  @param idn  - node number
  @param p    - pointer on problem
  @param r    - answer - allocated array for displacement
   
  @return The function returns require vector of nodal displacements in the array r.

  Created JK, 11.10.2003
*/
void noddispl (long lcid, long idn, problem *p, double *r)
{
  long i,ii;
  
  switch (p->mp->tprob)
  {
  case forced_dynamics:
  case growing_mech_structure://tady??!!
  case mech_timedependent_prob:
    {
      for (i=0;i<p->gt->give_ndofn (idn);i++){
        ii = p->gt->give_dof(idn,i);
        if (ii<0)   r[i]=p->mb->dlc[lcid].get_pd(p->mp->time, ii);
        if (ii==0)  r[i]=0.0;
        if (ii>0)   r[i]=p->lsrs->lhs[lcid*p->lsrs->ndof+ii-1];
      }
      break;
    }
    case mat_nonlinear_statics:
    {
      for (i=0;i<p->gt->give_ndofn (idn);i++){
        ii = p->gt->give_dof(idn,i);
        if (ii<0)   r[i]=p->mb->lc[lcid].pd[0-ii-1] * p->mp->lambda + p->mb->lc[lcid+1].pd[0-ii-1];
        if (ii==0)  r[i]=0.0;
        if (ii>0)   r[i]=p->lsrs->lhs[lcid*p->lsrs->ndof+ii-1];
      }
      break;
    }
    default:
    {
      for (i=0;i<p->gt->give_ndofn (idn);i++)
      {
         ii = p->gt->give_dof(idn,i);
        if (ii<0)   r[i]=p->mb->lc[lcid].pd[0-ii-1];
        if (ii==0)  r[i]=0.0;
        if (ii>0)   r[i]=p->lsrs->lhs[lcid*p->lsrs->ndof+ii-1];
      }
    }
  }
}



/**
  The function assembles constraint %matrix of one layer on one node.
   
  @param nid - node id
  @param cid - contact id
  @param lcm - local constraint %matrix (output)
   
  @return The function returns constraint %matrix in the parameter lcm.

  Created by JK, 7.12.2002
*/
void constr_matrix (long nid,long cid,matrix &lcm)
{
  long i,idcs;
  crsectype crst;
  double tl,tu;
  
  i=Gtm->lgnodes[nid].nodes[cid-1];
  crst = Mt->nodes[i].crst;
  idcs = Mt->nodes[i].idcs;
  tl = Mc->give_onethickness (crst,idcs);
  i=Gtm->lgnodes[nid].nodes[cid];
  crst = Mt->nodes[i].crst;
  idcs = Mt->nodes[i].idcs;
  tu = Mc->give_onethickness (crst,idcs);
  
  fillm (0.0,lcm);

  if (Mp->tlm==1){
    lcm[0][0] = -1.0;
    lcm[1][1] = -1.0;
    lcm[2][0] = tl/(-2.0);
    
    lcm[3][0] = 1.0;
    lcm[4][1] = 1.0;
    lcm[5][0] = tu/(-2.0);
  }
  
  if (Mp->tlm==2){
    
    /*
      uplne stara verze
      lcm[0][0] = -1.0;
      lcm[1][1] = -1.0;
      lcm[2][2] = -1.0;
      lcm[1][3] = tl/2.0;
      lcm[0][4] = tl/(-2.0);
      lcm[3][5] = -1.0;
      
      lcm[0][6]  = 1.0;
      lcm[1][7]  = 1.0;
      lcm[2][8]  = 1.0;
      lcm[1][9]  = tu/2.0;
      lcm[0][10] = tu/(-2.0);
      lcm[3][11] = 1.0;
    */
    
    
    lcm[0][0] = -1.0;
    lcm[1][1] = -1.0;
    lcm[2][2] = -1.0;
    lcm[3][1] = tl/2.0;
    lcm[4][0] = tl/(-2.0);
    lcm[5][3] = -1.0;
    
    lcm[6][0]  = 1.0;
    lcm[7][1]  = 1.0;
    lcm[8][2]  = 1.0;
    lcm[9][1]  = tu/2.0;
    lcm[10][0] = tu/(-2.0);
    lcm[11][3] = 1.0;
  }
}



/**
  Function assembles right hand side of system of algebraic equations.
  It is created by prescribed displacements, forces, moments, eigenstrains, etc.
  and it contains nodal forces.
  In case that the flv is not NULL, %vector of load caused by forces only is stored in flv 
   
  Array rhs has to be cleaned in this function, called functions
  do not clean the array rhs.
   
  @param lcid - load case id
  @param rhs - array of right hand side
  @param flv - array of load %vector caused by forces only (default value is NULL)
   
  @return The function returns assembled vectors of righ hand side and load vector caused by force.

  Created by JK,
  Modified by Tomas Koudelka,
*/
void mefel_right_hand_side (long lcid,double *rhs,double *flv)
{
  if (flv)
    nullv(flv,Ndofm);

  switch (Mp->tprob){
  case linear_statics:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+lcid*Ndofm,Ndofm);
    Mb->lc[lcid].assemble (lcid,rhs+lcid*Ndofm,flv,1.0);
    break;
  }
  case eigen_dynamics:{
    break;
  }
  case forced_dynamics:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+lcid*Ndofm,Ndofm);
    //Mb->lc[lcid].assemble (lcid,av);
    Mb->dlc[lcid].assemble (lcid,rhs,flv,Ndofm,Mp->time);
    break;
  }
  case linear_stability:{
    break;
  }
  case mat_nonlinear_statics:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+(lcid*2+0)*Ndofm, Ndofm);
    nullv (rhs+(lcid*2+1)*Ndofm, Ndofm);
    Mb->lc[lcid*2+0].assemble (lcid*2+0,rhs+(lcid*2+0)*Ndofm,flv,1.0);
    Mb->lc[lcid*2+1].assemble (lcid*2+1,rhs+(lcid*2+1)*Ndofm,flv,1.0);
    break;
  }
  case geom_nonlinear_statics:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+(lcid*2+0)*Ndofm, Ndofm);
    nullv (rhs+(lcid*2+1)*Ndofm, Ndofm);
    Mb->lc[lcid*2+0].assemble (lcid*2+0,rhs+(lcid*2+0)*Ndofm,flv,1.0);
    Mb->lc[lcid*2+1].assemble (lcid*2+1,rhs+(lcid*2+1)*Ndofm,flv,1.0);
    break;
  }
  case mech_timedependent_prob:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+lcid*Ndofm, Ndofm);
    Mb->dlc[lcid].assemble (lcid,rhs+lcid*Ndofm,flv,Ndofm,Mp->time);
    break;
  }
  case growing_mech_structure:{
    //  lcid must be equal to zero

    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs, Ndofm);
    Mb->dlc[lcid].assemble (0,rhs,flv,Ndofm,Mp->time);
    break;
  }
  case earth_pressure:{
    
    break;
  }
  case layered_linear_statics:{
    
    break;
  }
  case lin_floating_subdomain:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+lcid*Ndofm,Ndofm);
    Mb->lc[lcid].assemble (lcid,rhs+lcid*Ndofm,flv,1.0);
    break;
  }
  case nonlin_floating_subdomain:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+(lcid*2+0)*Ndofm, Ndofm);
    nullv (rhs+(lcid*2+1)*Ndofm, Ndofm);
    Mb->lc[lcid*2+0].assemble (lcid*2+0,rhs+(lcid*2+0)*Ndofm,flv,1.0);
    Mb->lc[lcid*2+1].assemble (lcid*2+1,rhs+(lcid*2+1)*Ndofm,flv,1.0);
    break;
  }
    
  case hemivar_inequal:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+lcid*Ndofm,Ndofm);
    Mb->lc[lcid].assemble (lcid,rhs+lcid*Ndofm,flv,1.0);
    break;
  }
  case load_balancing:{
    //  components of the vector rhs must be set to zero
    //  function assemble adds contributions to the input vector
    nullv (rhs+lcid*Ndofm,Ndofm);
    Mb->lc[lcid].assemble (lcid,rhs+lcid*Ndofm,flv,1.0);
    break;
  }
    
  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  if (Mp->eigstrains==1){
    //  contribution to the right hand side (vector of nodal forces)
    //  from prescribed eigenstrains
    
    //  type of eigenstrain is prescribed eigenstrain
    Mm->est=eigstrain;
    
    //  computation of nodal forces caused by eigenstrains
    nodal_eigstrain_forces (lcid,rhs+lcid*Ndofm);
  }
}



/**
  The function computes all required values
  this function is called e.g. before output print.

  @param lcid - load case id

  @return The function does not return anything.

  Created by JK, 1.4.2004
  Modified by TKo, 07.2008
*/
void compute_req_val (long lcid)
{
  // strain computaion
  if (Mp->straincomp == 1){
    if (Mp->strainstate==0)
      compute_ipstrains (lcid);
    if (Mp->strainpos == 2)
      compute_nodestrains (lcid);
  }
  
  // stress computaion
  if (Mp->stresscomp == 1){
    // strains have to be computed if they are not computed yet
    if (Mp->strainstate==0)
      compute_ipstrains (lcid);
    if (Mp->stressstate==0)
      compute_ipstresses (lcid);
    if (Mp->stresspos == 2)
      compute_nodestresses (lcid);
  }
  
  // other values computation
  if ((Mp->othercomp == 1) && (Mp->otherpos == 2) &&(Mp->otherstate==1))
    compute_nodeothers (lcid);
  
  //  reactions computation
  if (Mp->reactcomp==1){
    // strains have to be computed at integration points
    if (Mp->strainstate==0)
      compute_ipstrains (lcid);
    switch (Mp->tprob){
    case linear_statics:
    case load_balancing:
    case mat_nonlinear_statics:
    case earth_pressure:
      Mb->lc[lcid].compute_reactions (lcid);
      break;
    case forced_dynamics:
    case mech_timedependent_prob:
    case growing_mech_structure:
      Mb->dlc[lcid].compute_reactions(lcid);
      break;
    case eigen_dynamics:
      break; 
    default:
      print_err("unknown type of problem is required",__FILE__,__LINE__,__func__);
    }
  }

  // computaion of strains have to be last due to total strains
  // all previous computations need strains - eigenstrains
  if (Mp->straincomp == 1){
    if (Mp->strainstate==0)
      compute_ipstrains (lcid);
    // in previous statements, strains were modified
    // such that the eigstrains and temprstrains were subtracted
    // total strains have to be computed again    
    Mm->totstrains ();
    if (Mp->strainpos == 2)
      compute_nodestrains (lcid);
  }
}



/**
  The function transforms nodal displacements form the nodal local coordinate system
  to the global one.

  @param lcid - load case id

  @return The function does not return anything.

  Created by JK,
*/
void local_global_displ_transf (long lcid)
{
  long i,j,k,ndofn;
  long *cn;
  vector l,g;
  matrix tm(3,3);
  
  for (i=0;i<Mt->nn;i++){
    if (Mt->nodes[i].transf>0){

      //fprintf (Outm,"\n uzel %ld   %ld",i,Mt->nodes[i].transf);

      ndofn = Mt->give_ndofn (i);
      allocv (ndofn,l);
      allocv (ndofn,g);
      
      noddispl (lcid,l.a,i);

      tm[0][0]=Mt->nodes[i].e1[0];
      tm[1][0]=Mt->nodes[i].e1[1];
      tm[2][0]=Mt->nodes[i].e1[2];
      
      tm[0][1]=Mt->nodes[i].e2[0];
      tm[1][1]=Mt->nodes[i].e2[1];
      tm[2][1]=Mt->nodes[i].e2[2];
      
      tm[0][2]=Mt->nodes[i].e3[0];
      tm[1][2]=Mt->nodes[i].e3[1];
      tm[2][2]=Mt->nodes[i].e3[2];
      
      mxv(tm,l,g);
      
      cn = new long [ndofn];
      Mt->give_node_code_numbers (i,cn);
      
      for (j=0;j<ndofn;j++){
	k=cn[j]-1;
	if (k>-1)
	  Lsrs->lhs[k]=g[j];
      }
      delete [] cn;
      destrv (l);
      destrv (g);
    }
  }
}



/**
  The function assembles right hand side %vector
  used due to correct graphics representation of the deformed shape 
  in case of the growing structures.

  Parameters:
  @param rhs - righ hand side %vector (output)
  @param lcid - load case id
  @param n - number of components of rhs

  Created by JK,
*/
void rhs_graphic (double *rhs,long lcid,long n)
{
  long i,j,k,m,ndofe,nne,ndofn;
  long *cn;
  ivector nodes;
  vector r,f;
  matrix sm;
  
  nullv (rhs,n);
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      
      ndofe=Mt->give_ndofe (i);
      nne=Mt->give_nne (i);
      allocv (nne,nodes);
      Mt->give_elemnodes (i,nodes);
      
      allocm (ndofe,ndofe,sm);
      stiffmat (lcid,i,sm);
      
      allocv (ndofe,r);
      allocv (ndofe,f);
      cn = new long [ndofe];
      Mt->give_code_numbers (i,cn);
      for (j=0;j<ndofe;j++){
	r[j]=Mt->elements[i].initdispl[j];
      }
      
      m=0;
      for (j=0;j<nne;j++){
	ndofn=Mt->give_ndofn (nodes[j]);
	for (k=0;k<ndofn;k++){
	  r[m]+=Mt->nodes[nodes[j]].nvgraph[k];
	  m++;
	}
      }
      
      mxv (sm,r,f);
      cmulv (-1.0,f);
      locglob (rhs,f.a,cn,ndofe);
      
      delete [] cn;
      destrm (sm);
      destrv (f);  destrv (r);
      destrv (nodes);
    }
  }
}

