#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include "mechtop.h"
#include "global.h"
#include "elemhead.h"
#include "gtopology.h"
#include "difcalc.h"
#include "mathem.h"
#include "element.h"
#include "node.h"
#include "endnodem.h"
#include "edgem.h"
#include "siftop_element_types.h"
#include "loadcase.h"
#include "intpoints.h"
#include "globmat.h"
#include "siftop.h"
#include "saddpoint.h"



/**
  Constructor initializes data members to zero values.

  Created by JK,
*/
mechtop::mechtop ()
{
  //  number of nodes
  nn=0;
  //  number of constrained nodes
  ncn=0;
  //  number of elements
  ne=0;
  //  number of layered nodes
  nln=0;
  
  //  node array
  nodes=NULL;
  //  element array
  elements=NULL;
  
  dist=NULL;
  nadjip=NULL;
  adjip=NULL;
  nodedispl=NULL;
}



/**
  Destructor deallocates used memory.

  Created by JK
  Modified by TKo
*/
mechtop::~mechtop ()
{
  long i;
  
  delete [] nodes;  
  delete [] elements;
  delete [] nadjip;
  if (adjip)
  {
  for (i=0; i<Mm->tnip; i++)  
  delete [] adjip[i];  
  }
  if (dist)
  {
    for (i=0; i<Mm->tnip; i++)  
      delete [] dist[i];
  }
  delete [] adjip;  
  delete [] dist;
  
  if (nodedispl != NULL){
    for (i=0;i<nn;i++){
      delete [] nodedispl[i];
    }
    delete [] nodedispl;
  }
}



/**
  Function reads basic topology informations.

  @param in - input stream

  @return The function does not return anything.

  Created by JK,
*/
void mechtop::read (XFILE *in)
{
  long i,j,k,l,ndofn,nne,dim,order;
  
  // ********************
  //  node data reading
  // ********************
  xfscanf (in,"%k%ld","number_of_nodes",&nn);
  Gtm->alloc_nodes (nn);

  if (Mespr==1)  fprintf (stdout,"\n number of nodes  %ld",nn);
  nodes = new node [nn];
  
  switch (Mp->tprob){
  case linear_statics:
  case eigen_dynamics:
  case forced_dynamics:
  case linear_stability:
  case mat_nonlinear_statics:
  case geom_nonlinear_statics:
  case mech_timedependent_prob:
  case growing_mech_structure:
  case load_balancing:
  case lin_floating_subdomain:
  case hemivar_inequal:
  case earth_pressure:{
    for (i=0;i<nn;i++){
      j=nn+1;
      xfscanf (in,"%ld",&j);
      if (j<1)
	print_err("node number in function read is less than 1",__FILE__,__LINE__,__func__);
      if (j>nn)
	print_err("node number in function read is greater than number of nodes",__FILE__,__LINE__,__func__);
      Gtm->gnodes[j-1].read (in);
      ndofn = Gtm->gnodes[j-1].ndofn;
      nodes[j-1].read (in,ndofn);
    }
    break;
  }
  case layered_linear_statics:{
    xfscanf (in,"%k%ld","number_of_layered_nodes",&nln);
    Gtm->alloc_lnodes (nln);
    for (i=0;i<nln;i++){
      xfscanf (in,"%ld",&l);
      xfscanf (in,"%ld",&(Gtm->lgnodes[l-1].nl));
      Gtm->lgnodes[l-1].nodes = new long [Gtm->lgnodes[l-1].nl];
      for (k=0;k<Gtm->lgnodes[l-1].nl;k++){
	j=nn+1;
	xfscanf (in,"%ld",&j);
	if (j<1)
	  print_err("node number in function read is less than 1",__FILE__,__LINE__,__func__);
	if (j>nn)
	  print_err("node number in function read is greater than number of nodes",__FILE__,__LINE__,__func__);
	Gtm->gnodes[j-1].read (in);
	ndofn = Gtm->gnodes[j-1].ndofn;
	nodes[j-1].read (in,ndofn);
	Gtm->lgnodes[l-1].nodes[k]=j-1;
      }
    }
    Gtm->unodelnode ();
    break;
  }

  case nonlin_floating_subdomain:{
    //  number of subdomains
    xfscanf (in,"%k%ld","number_of_floating_subdomains",&(Gtm->flsub.nsd));
    //  allocation of arrays
    Gtm->flsub.nnsd = new long [Gtm->flsub.nsd];
    Gtm->flsub.fnnsd = new long [Gtm->flsub.nsd+1];
    
    Gtm->flsub.fnnsd[0]=0;
    for (i=0;i<Gtm->flsub.nsd;i++){
      xfscanf (in,"%ld",&(Gtm->flsub.nnsd[i]));
      Gtm->flsub.fnnsd[i+1] = Gtm->flsub.fnnsd[i]+Gtm->flsub.nnsd[i];
    }
    Gtm->flsub.fnnsd[Gtm->flsub.nsd]--;

    //  nodes
    for (i=0;i<nn;i++){
      j=nn+1;
      xfscanf (in,"%ld",&j);
      if (j<1)
	print_err("node number in function read is less than 1",__FILE__,__LINE__,__func__);
      if (j>nn)
	print_err("node number in function read is greater than number of nodes",__FILE__,__LINE__,__func__);
      Gtm->gnodes[j-1].read (in);
      ndofn = Gtm->gnodes[j-1].ndofn;
      nodes[j-1].read (in,ndofn);
    }
    
    //  layered nodes
    xfscanf (in,"%ld",&nln);
    Gtm->alloc_lnodes (nln);
    for (i=0;i<nln;i++){
      xfscanf (in,"%ld",&l);
      xfscanf (in,"%ld",&(Gtm->lgnodes[l-1].nl));
      Gtm->lgnodes[l-1].nodes = new long [Gtm->lgnodes[l-1].nl];
      for (k=0;k<Gtm->lgnodes[l-1].nl;k++){
	j=nn+1;
	xfscanf (in,"%ld",&j);
	if (j<1)
	  print_err("node number in function read is less than 1",__FILE__,__LINE__,__func__);
	if (j>nn)
	  print_err("node number in function read is greater than number of nodes",__FILE__,__LINE__,__func__);
	Gtm->lgnodes[l-1].nodes[k]=j-1;
      }
    }
    //Gtm->unodelnode ();
    break;
  }
    
  default:{
    print_err("unknown type of problem is required",__FILE__,__LINE__,__func__);
  }
  }
  
  //  constrained nodes
  xfscanf (in,"%k%ld","number_of_constraints",&ncn);
  if (Mespr==1)  fprintf (stdout,"\n number of constrained nodes  %ld",ncn);
  for (i=0;i<ncn;i++){
    k=nn+1;
    xfscanf (in,"%ld",&k);
    if (k<1)
      print_err("number of constrained node in function read is less than 1",__FILE__,__LINE__,__func__);
    if (k>nn){
      print_err("number of constrained node in function read is greater than number of all nodes",__FILE__,__LINE__,__func__);
    }
    Gtm->gnodes[k-1].constr (Gtm->dofcontr,in);
  }

  // ***********************
  // ***********************
  //  element data reading
  // ***********************
  // ***********************
  xfscanf (in,"%k%ld","number_of_elements",&ne);
  Gtm->alloc_elements (ne);
  if (Mespr==1)  fprintf (stdout,"\n number of elements  %ld",ne);
  elements = new element [ne];
  
  switch (Mp->tprob){
  case linear_statics:
  case eigen_dynamics:
  case forced_dynamics:
  case linear_stability:
  case mat_nonlinear_statics:
  case geom_nonlinear_statics:
  case mech_timedependent_prob:
  case growing_mech_structure:
  case earth_pressure:
  case lin_floating_subdomain:
  case hemivar_inequal:
  case load_balancing:
  case layered_linear_statics:{
    for (i=0;i<ne;i++){
      j=ne+1;
      xfscanf (in,"%ld",&j);
      if (j<1)
	print_err("element number in function read is less than 1",__FILE__,__LINE__,__func__);
      if (j>ne){
	print_err("element number in function read is greater than total number of elements",__FILE__,__LINE__,__func__);
      }
      elements[j-1].read (in,j-1);
    }
    break;
  }

  case nonlin_floating_subdomain:{
    Gtm->flsub.nesd = new long [Gtm->flsub.nsd];
    Gtm->flsub.fensd = new long [Gtm->flsub.nsd+1];
    Gtm->flsub.fensd[0]=0;
    //  number of elements on particular subdomains
    for (i=0;i<Gtm->flsub.nsd;i++){
      xfscanf (in,"%ld",&(Gtm->flsub.nesd[i]));
      Gtm->flsub.fensd[i+1]=Gtm->flsub.fensd[i]+Gtm->flsub.nesd[i];
    }
    
    if (ne!=Gtm->flsub.fensd[Gtm->flsub.nsd]){
      print_err("the sum of numbers of elements on subdomains\n is not equal to the total number of elements", __FILE__, __LINE__, __func__);
    }
    
    for (i=0;i<ne;i++){
      j=ne+1;
      xfscanf (in,"%ld",&j);
      if ((j<1) || (j>ne))
	print_err ("element number is out of range <1,%ld>.",__FILE__,__LINE__, __func__, ne);
      elements[j-1].read (in,j-1);
    }
    break;
  }
  default:{
    print_err("unknown type of problem is required",__FILE__,__LINE__,__func__);
  }
  }
  
  /*
  if (Mp->tprob==layered_linear_statics){
    fscanf (in,"%ld",&nle);
    Gtm->alloc_lelements (nle);
    for (i=0;i<nle;i++){
      fscanf (in,"%ld",&l);
      fscanf (in,"%ld",&(Gtm->lgelements[l-1].nl));
      Gtm->lgelements[l-1].elem = new long [Gtm->lgelements[l-1].nl];
      for (k=0;k<Gtm->lgelements[l-1].nl;k++){
	j=ne+1;
	fscanf (in,"%ld",&j);
	if (j<1)
	  fprintf (stderr,"\n\n element number in function mechtop::read is less than 1.\n");
	if (j>ne){
	  fprintf (stderr,"\n\n element number in function mechtop::read is");
	  fprintf (stderr,"\n greater than total number of elements.\n");
	}
	elements[j-1].read (in,j-1);
	Gtm->lgelements[l-1].elem[k]=j-1;
      }
      Gtm->lgelements[l-1].elemnmult ();
    }
  }
  else{
  */

  if (Mp->adaptivity > 0){
    //  initialization of auxinf variable
    for (i=0;i<ne;i++){
      nne = give_nne (i);
      order = give_degree (i);
      dim = give_dimension (i);
      
      Gtm->gelements[i].auxinf = nne*100+order*10+dim;
    }
  }
  
  //  allocation and initiation of arrays lnso and leso
  //  lnso - list of nodes switched on
  //  leso - list of elements switched on
  //  3.11.2006
  Gtm->lneso_init ();

  if (Mp->tprob == growing_mech_structure){
    Gtm->auxinf_init ();
  }
  
  if (Gtm->rst==1){
    //  sequential topology is read
    //  it is used in the case of sequential version of
    //  any domain decomposition method
    Gtm->read_seq_top (in);
  }
  if (Gtm->rst==2){
    //  sequential topology is read in the form
    //  of Boolean matrices
    Gtm->read_seq_top (in);
    Mp->ssle->feti.read_booldata (in);
  }
  
}



/**
  Function investigates presence of prescribed displacements on elements.
  
  @return The function does not return anything.
 
  Created by JK, 25.7.2001
*/
void mechtop::elemprescdisp (void)
{
  long i,j,ndofe,*cn;
  
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      ndofe=give_ndofe (i);
      cn = new long [ndofe];
      give_code_numbers (i,cn);
      for (j=0;j<ndofe;j++){
	if (cn[j]<0){
	  elements[i].prescdispl=1;
	  break;
	}
      }
      delete [] cn;
    }
  }
}



/**
  Function investigates presence of temperature changes on elements.
  
  @param lcid - load case id

  @return The function does not return anything.
  
  Created by JK, 30.11.2002
*/
void mechtop::elempresctemp (long lcid)
{
  long i;
  ivector nod;

  if (Mp->temperature==1 || Mp->temperature==2){
    for (i=0;i<ne;i++){
      if (Gtm->leso[i]==1){
	elements[i].presctemp=1;
      }
    }
  }
  
  /*
  if (Mb->lc[lcid].tempchang==1){
    for (i=0;i<ne;i++){
      nne=give_nne (i);
      allocv (nne,nod);
      dtelem = new double [nne];
      give_elemnodes (i,nod);
      Mb->lc[lcid].tempchanges (dtelem,nod);
      for (j=0;j<nne;j++){
	if (dtelem[j]!=0.0){
	  elements[i].presctemp=1;
	  break;
	}
      }
      delete [] dtelem;
      destrv (nod);
    }
  }
*/
}



/**
  Function returns element type.
  
  @param eid - element id

  @return The function returns type of the required element.

  Created by JK,
*/
elemtype mechtop::give_elem_type (long eid)
{
  return (elements[eid].te);
}



/**
  Function returns number of DOFs of the element.

  @param eid - element id

  @return The function returns number of DOFs of the required element.

  Created by JK,
*/
long mechtop::give_ndofe (long eid)
{
  long ndofe=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:{               ndofe=Bar2d->ndofe;              break; }
  case bar3d:{               ndofe=Bar3d->ndofe;              break; }
  case barq2d:{              ndofe=Barq2d->ndofe;             break; }
  case barq3d:{              ndofe=Barq3d->ndofe;             break; }
  case beam2d:{              ndofe=Beam2d->ndofe;             break; }
  case beam3d:{              ndofe=Beam3d->ndofe;             break; }
  case beamg3d:{             ndofe=Beam3dg->ndofe;            break; }
  case subsoilbeam:{         ndofe=Sbeam->ndofe;              break; }
  case beam2dsp:{            ndofe=Spbeam2d->ndofe;           break; }
  case spring_1:{            ndofe=Spring->give_ndofe(eid);   break; }
  case spring_2:{            ndofe=Spring->give_ndofe(eid);   break; }
  case spring_3:{            ndofe=Spring->give_ndofe(eid);   break; }
  case spring_4:{            ndofe=Spring->give_ndofe(eid);   break; }
  case spring_5:{            ndofe=Spring->give_ndofe(eid);   break; }
  case spring_6:{            ndofe=Spring->give_ndofe(eid);   break; }
  case planeelementlt:{      ndofe=Pelt->ndofe;               break; }
  case planeelementqt:{      ndofe=Peqt->ndofe;               break; }
  case planeelementrotlt:{   ndofe=Perlt->ndofe;              break; }
  case planeelementlq:{      ndofe=Pelq->ndofe;               break; }
  case planeelementqq:{      ndofe=Peqq->ndofe;               break; }
  case planeelementrotlq:{   ndofe=Perlq->ndofe;              break; }
  case planeelementsubqt:{   ndofe=Pesqt->ndofe;              break; }
  case planequadcontact:{    ndofe=Pqcon->ndofe;              break; }
  case cctel:{               ndofe=Cct->ndofe;                break; }
  case dktel:{               ndofe=Dkt->ndofe;                break; }
  case dstel:{               ndofe=Dst->ndofe;                break; }
  case q4plateel:{           ndofe=Q4pl->ndofe;               break; }
  case subsoilplatetr:{      ndofe=Spltr->ndofe;              break; }
  case subsoilplateq:{       ndofe=Splq->ndofe;               break; }
  case axisymmlt:{           ndofe=Asymlt->ndofe;             break; }
  case axisymmlq:{           ndofe=Asymlq->ndofe;             break; }
  case axisymmqq:{           ndofe=Asymqq->ndofe;             break; }
  case shelltrelem:{         ndofe=Shtr->ndofe;               break; }
  case shellqelem:{          ndofe=Shq->ndofe;                break; }
  case lineartet:{           ndofe=Ltet->ndofe;               break; }
  case quadrtet:{            ndofe=Qtet->ndofe;               break; }
  case linearhex:{           ndofe=Lhex->ndofe;               break; }
  case quadrhex:{            ndofe=Qhex->ndofe;               break; }
  case lineartetrot:{        ndofe=Ltetrot->ndofe;            break; }
  case linearhexrot:{        ndofe=Lhexrot->ndofe;            break; }
  case linearwed:{           ndofe=Lwed->ndofe;               break; }
  case quadrwed:{            ndofe=Qwed->ndofe;               break; }
  case particleelem:{        ndofe=Pelem->ndofe;              break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return ndofe;
}



/**
  Function returns number of DOFs of node.

  @param nid - node id

  @return The function returns number of DOFs of the required node.

  Created by JK,
*/
long mechtop::give_ndofn (long nid)
{
  return Gtm->give_ndofn (nid);
}



/**
  Function returns appropriate DOF of node.

  @param nid - node id
  @param n - number of required DOF

  @return The function returns DOF of the required node.

  Created by JK,
*/
long mechtop::give_dof (long nid,long n)
{
  return Gtm->give_dof (nid,n);
}



/**
  Function returns number of nodes of the element.
  
  @param eid - element id

  @return The function returns number of nodes of the required element.

  Created by JK,
*/
long mechtop::give_nne (long eid)
{
  long nne;
  elemtype te;
  
  te = give_elem_type (eid);
  nne = give_nne_inner (te);
  return nne;
}



/**
  Function returns number nodes on the given element type.

  @param te - element type

  @return The function returns number of nodes of element with given type.
   
  Created by JK, 27.2.2002
*/
long mechtop::give_nne_inner (elemtype te)
{
  long nne=0;

  switch (te){
  case bar2d:{              nne=Bar2d->nne;     break; }
  case bar3d:{              nne=Bar3d->nne;     break; }
  case barq2d:{             nne=Barq2d->nne;    break; }
  case barq3d:{             nne=Barq3d->nne;    break; }
  case beam2d:{             nne=Beam2d->nne;    break; }
  case beam3d:{             nne=Beam3d->nne;    break; }
  case beamg3d:{            nne=Beam3dg->nne;   break; }
  case subsoilbeam:{        nne=Sbeam->nne;     break; }
  case beam2dsp:{           nne=Spbeam2d->nne;  break; }
  case spring_1:{           nne=Spring->nne;    break; }
  case spring_2:{           nne=Spring->nne;    break; }
  case spring_3:{           nne=Spring->nne;    break; }
  case spring_4:{           nne=Spring->nne;    break; }
  case spring_5:{           nne=Spring->nne;    break; }
  case spring_6:{           nne=Spring->nne;    break; }
  case planeelementlt:{     nne=Pelt->nne;      break; }
  case planeelementqt:{     nne=Peqt->nne;      break; }
  case planeelementrotlt:{  nne=Perlt->nne;     break; }
  case planeelementlq:{     nne=Pelq->nne;      break; }
  case planeelementqq:{     nne=Peqq->nne;      break; }
  case planeelementrotlq:{  nne=Perlq->nne;     break; }
  case planeelementsubqt:{  nne=Pesqt->nne;     break; }
  case planequadcontact:{   nne=Pqcon->nne;     break; }
  case cctel:{              nne=Cct->nne;       break; }
  case dktel:{              nne=Dkt->nne;       break; }
  case dstel:{              nne=Dst->nne;       break; }
  case q4plateel:{          nne=Q4pl->nne;      break; }
  case subsoilplatetr:{     nne=Spltr->nne;     break; }
  case subsoilplateq:{      nne=Splq->nne;      break; }
  case axisymmlt:{          nne=Asymlt->nne;    break; }
  case axisymmlq:{          nne=Asymlq->nne;    break; }
  case axisymmqq:{          nne=Asymqq->nne;    break; }
  case shelltrelem:{        nne=Shtr->nne;      break; }
  case shellqelem:{         nne=Shq->nne;       break; }
  case lineartet:{          nne=Ltet->nne;      break; }
  case quadrtet:{           nne=Qtet->nne;      break; }
  case linearhex:{          nne=Lhex->nne;      break; }
  case quadrhex:{           nne=Qhex->nne;      break; }
  case lineartetrot:{       nne=Ltetrot->nne;   break; }
  case linearhexrot:{       nne=Lhexrot->nne;   break; }
  case linearwed:{          nne=Lwed->nne;      break; }
  case quadrwed:{           nne=Qwed->nne;      break; }
  case particleelem:{       nne=Pelem->nne;     break; }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__, __func__);
  }
  }
  return nne;
}



/**
  Function returns number of integration points.
    
  @param eid - element id
  @param ri,ci - row and column indices
  
  @return The function returns number of integration points of the given element block.

  Created by JK, 30.12.2001
*/
long mechtop::give_nip (long eid,long ri,long ci)
{
  long nip=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){

  case bar2d:{              nip=Bar2d->nip[ri][ci];      break;  }
  case bar3d:{              nip=Bar3d->nip[ri][ci];      break;  }
  case barq2d:{             nip=Barq2d->nip[ri][ci];     break;  }
  case barq3d:{             nip=Barq3d->nip[ri][ci];     break;  }
  case beam2d:{             nip=Beam2d->nip[ri][ci];     break;  }
  case beam3d:{             nip=Beam3d->nip[ri][ci];     break;  }
  case beamg3d:{            nip=Beam3dg->nip[ri][ci];    break;  }
  case subsoilbeam:{        nip=Sbeam->nip[ri][ci];      break;  }
  case beam2dsp:{           nip=Spbeam2d->nip[ri][ci];   break;  }
  case spring_1:{           nip=Spring->nip[ri][ci];     break;  }
  case spring_2:{           nip=Spring->nip[ri][ci];     break;  }
  case spring_3:{           nip=Spring->nip[ri][ci];     break;  }
  case spring_4:{           nip=Spring->nip[ri][ci];     break;  }
  case spring_5:{           nip=Spring->nip[ri][ci];     break;  }
  case spring_6:{           nip=Spring->nip[ri][ci];     break;  }
  case planeelementlt:{     nip=Pelt->nip[ri][ci];       break;  }
  case planeelementqt:{     nip=Peqt->nip[ri][ci];       break;  }
  case planeelementrotlt:{  nip=Perlt->nip[ri][ci];      break;  }
  case planeelementlq:{     nip=Pelq->nip[ri][ci];       break;  }
  case planeelementqq:{     nip=Peqq->nip[ri][ci];       break;  }
  case planeelementrotlq:{  nip=Perlq->nip[ri][ci];      break;  }
  case planeelementsubqt:{  nip=Pesqt->nip[ri][ci];      break;  }
  case planequadcontact:{   nip=Pqcon->nip[ri][ci];      break;  }
  case cctel:{              nip=Cct->nip[ri][ci];        break;  }
  case dktel:{              nip=Dkt->nip[ri][ci];        break;  }
  case dstel:{              nip=Dst->nip[ri][ci];        break;  }
  case q4plateel:{          nip=Q4pl->nip[ri][ci];       break;  }
  case subsoilplatetr:{     nip=Spltr->nip[ri][ci];      break;  }
  case subsoilplateq:{      nip=Splq->nip[ri][ci];       break;  }
  case axisymmlt:{          nip=Asymlt->nip[ri][ci];     break;  }
  case axisymmlq:{          nip=Asymlq->nip[ri][ci];     break;  }
  case axisymmqq:{          nip=Asymqq->nip[ri][ci];     break;  }
  case shelltrelem:{        nip=Shtr->nip[ri][ci];       break;  }
  case shellqelem:{         nip=Shq->nip[ri][ci];        break;  }
  case lineartet:{          nip=Ltet->nip[ri][ci];       break;  }
  case quadrtet:{           nip=Qtet->nip[ri][ci];       break;  }
  case linearhex:{          nip=Lhex->nip[ri][ci];       break;  }
  case quadrhex:{           nip=Qhex->nip[ri][ci];       break;  }
  case lineartetrot:{       nip=Ltetrot->nip[ri][ci];    break;  }
  case linearhexrot:{       nip=Lhexrot->nip[ri][ci];    break;  }
  case linearwed:{          nip=Lwed->nip[ri][ci];       break;  }
  case quadrwed:{           nip=Qwed->nip[ri][ci];       break;  }
  case particleelem:{       nip=Pelem->nip[ri][ci];      break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nip;
}



/**
  Function returns total number of integration points of the given element.
   
  @param eid - element id
   
  @return The function returns total number of integration points.

  Created by JK, 21.6.2004
*/
long mechtop::give_tnip (long eid)
{
  long tnip = -1;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:   {           tnip=Bar2d->tnip;      break;  }
  case bar3d:   {           tnip=Bar3d->tnip;      break;  }
  case barq2d:  {           tnip=Barq2d->tnip;     break;  }
  case barq3d:  {           tnip=Barq3d->tnip;     break;  }
  case beam2d:{             tnip=Beam2d->tnip;     break;  }
  case beam3d:{             tnip=Beam3d->tnip;     break;  }
  case subsoilbeam:{        tnip=Sbeam->tnip;      break;  }
  case beamg3d:{            tnip=Beam3dg->tnip;    break;  }
  case beam2dsp:{           tnip=Spbeam2d->tnip;   break;  }
  case spring_1:{           tnip=Spring->tnip;     break;  }
  case spring_2:{           tnip=Spring->tnip;     break;  }
  case spring_3:{           tnip=Spring->tnip;     break;  }
  case spring_4:{           tnip=Spring->tnip;     break;  }
  case spring_5:{           tnip=Spring->tnip;     break;  }
  case spring_6:{           tnip=Spring->tnip;     break;  }
  case planeelementlt:{     tnip=Pelt->tnip;       break;  }
  case planeelementqt:{     tnip=Peqt->tnip;       break;  }
  case planeelementrotlt:{  tnip=Perlt->tnip;      break;  }
  case planeelementlq:{     tnip=Pelq->tnip;       break;  }
  case planeelementqq:{     tnip=Peqq->tnip;       break;  }
  case planeelementrotlq:{  tnip=Perlq->tnip;      break;  }
  case planeelementsubqt:{  tnip=Pesqt->tnip;      break;  }
  case planequadcontact:{   tnip=Pqcon->tnip;      break;  }
  case cctel:{              tnip=Cct->tnip;        break;  }
  case dktel:{              tnip=Dkt->tnip;        break;  }
  case dstel:{              tnip=Dst->tnip;        break;  }
  case q4plateel:{          tnip=Q4pl->tnip;       break;  }
  case subsoilplatetr:{     tnip=Spltr->tnip;      break;  }
  case subsoilplateq:{      tnip=Splq->tnip;       break;  }
  case axisymmlt:{          tnip=Asymlt->tnip;     break;  }
  case axisymmlq:{          tnip=Asymlq->tnip;     break;  }
  case axisymmqq:{          tnip=Asymqq->tnip;     break;  }
  case shelltrelem:{        tnip=Shtr->tnip;       break;  }
  case shellqelem:{         tnip=Shq->tnip;        break;  }
  case lineartet:{          tnip=Ltet->tnip;       break;  }
  case quadrtet:{           tnip=Qtet->tnip;       break;  }
  case linearhex:{          tnip=Lhex->tnip;       break;  }
  case quadrhex:{           tnip=Qhex->tnip;       break;  }
  case lineartetrot:{       tnip=Ltetrot->tnip;    break;  }
  case linearhexrot:{       tnip=Lhexrot->tnip;    break;  }
  case linearwed:{          tnip=Lwed->tnip;       break;  }
  case quadrwed:{           tnip=Qwed->tnip;       break;  }
  case particleelem:{       tnip=1;                break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return tnip;
}



/**
  Function returns total number of integration points on element.
   
  @param eid - element id

  @return The function returns total number of integration points on element. 
   
  Created by Tomas Koudelka, 30.12.2001
*/
long mechtop::give_totnip (long eid)
{
  if (eid < Mt->ne-1)
    return elements[eid+1].ipp[0][0] - elements[eid].ipp[0][0];

  return Mm->tnip-elements[eid].ipp[0][0];
}



/**
  Function returns number of strain/stress components.
   
  @param eid - element id
  @param bid - block id
   
  @return The function returns number of stress/strain components

  Created by JK, 30.12.2001
*/
long mechtop::give_ncomp (long eid,long bid)
{
  long ncomp=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:{
    //ncomp=4;
    ncomp=Bar2d->tncomp;
    break;  }
  case bar3d:{              ncomp=Bar3d->tncomp;      break;  }
  case barq2d:{             ncomp=Barq2d->tncomp;     break;  }
  case barq3d:{             ncomp=Barq3d->tncomp;     break;  }
  case beam2d:{             ncomp=Beam2d->tncomp;     break;  }
  case beam3d:{             ncomp=Beam3d->tncomp;     break;  }
  case beamg3d:{            ncomp=Beam3dg->tncomp;    break;  }
  case beam2dsp:{           ncomp=Spbeam2d->tncomp;   break;  }
  case subsoilbeam:{        ncomp=Sbeam->tncomp;      break;  }
  case spring_1:{           ncomp=Spring->tncomp;     break;  }
  case spring_2:{           ncomp=Spring->tncomp;     break;  }
  case spring_3:{           ncomp=Spring->tncomp;     break;  }
  case spring_4:{           ncomp=Spring->tncomp;     break;  }
  case spring_5:{           ncomp=Spring->tncomp;     break;  }
  case spring_6:{           ncomp=Spring->tncomp;     break;  }
  case planeelementlt:{     ncomp=Pelt->tncomp;       break;  }
  case planeelementqt:{     ncomp=Peqt->tncomp;       break;  }
  case planeelementrotlt:{  ncomp=Perlt->tncomp;      break;  }
  case planeelementlq:{     ncomp=Pelq->tncomp;       break;  }
  case planeelementqq:{     ncomp=Peqq->tncomp;       break;  }
  case planeelementrotlq:{  ncomp=Perlq->tncomp;      break;  }
  case planeelementsubqt:{  ncomp=Pesqt->tncomp;      break;  }
  case planequadcontact:{   ncomp=Pqcon->tncomp;      break;  }
  case cctel:{              ncomp=Cct->tncomp;        break;  }
  case dktel:{              ncomp=Dkt->tncomp;        break;  }
  case dstel:{              ncomp=Dst->tncomp;        break;  }
  case q4plateel:{          ncomp=Q4pl->tncomp;       break;  }
  case subsoilplatetr:{     ncomp=Spltr->tncomp;      break;  }
  case subsoilplateq:{      ncomp=Splq->tncomp;       break;  }
  case axisymmlt:{          ncomp=Asymlt->tncomp;     break;  }
  case axisymmlq:{          ncomp=Asymlq->tncomp;     break;  }
  case axisymmqq:{          ncomp=Asymqq->tncomp;     break;  }
  case shelltrelem:{
    if (bid==0 || bid==1)   ncomp=Perlt->tncomp;
    if (bid==2)             ncomp=Dkt->tncomp;
    break;
  }
  case shellqelem:{
    if (bid==0 || bid==1)   ncomp=Perlq->tncomp;
    if (bid==2 || bid==3)   ncomp=Q4pl->tncomp;
    break;
  }
  case lineartet:{          ncomp=Ltet->tncomp;       break;  }
  case quadrtet:{           ncomp=Qtet->tncomp;       break;  }
  case linearhex:{          ncomp=Lhex->tncomp;       break;  }
  case quadrhex:{           ncomp=Qhex->tncomp;       break;  }
  case lineartetrot:{       ncomp=Ltetrot->tncomp;    break;  }
  case linearhexrot:{       ncomp=Lhexrot->tncomp;    break;  }
  case linearwed:{          ncomp=Lwed->tncomp;       break;  }
  case quadrwed:{           ncomp=Qwed->tncomp;       break;  }
  case particleelem:{       ncomp=Pelem->ndofe;       break;  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  return ncomp;
}



/**
  Function returns number of strain/stress components.
  ???!!! candidate for removal, above function seems identical
   
  @param eid - element id

  @return The function returns number of stress/strain components
     
  Created by JK, 1.10.2008
*/
long mechtop::give_tncomp (long eid)
{
  long tncomp=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:{              tncomp=Bar2d->tncomp;      break;  }
  case bar3d:{              tncomp=Bar3d->tncomp;      break;  }
  case barq2d:{             tncomp=Barq2d->tncomp;     break;  }
  case barq3d:{             tncomp=Barq3d->tncomp;     break;  }
  case beam2d:{             tncomp=Beam2d->tncomp;     break;  }
  case beam3d:{             tncomp=Beam3d->tncomp;     break;  }
  case beamg3d:{            tncomp=Beam3dg->tncomp;    break;  }
  case beam2dsp:{           tncomp=Spbeam2d->tncomp;   break;  }
  case subsoilbeam:{        tncomp=Sbeam->tncomp;      break;  }
  case spring_1:{           tncomp=Spring->tncomp;     break;  }
  case spring_2:{           tncomp=Spring->tncomp;     break;  }
  case spring_3:{           tncomp=Spring->tncomp;     break;  }
  case spring_4:{           tncomp=Spring->tncomp;     break;  }
  case spring_5:{           tncomp=Spring->tncomp;     break;  }
  case spring_6:{           tncomp=Spring->tncomp;     break;  }
  case planeelementlt:{     tncomp=Pelt->tncomp;       break;  }
  case planeelementqt:{     tncomp=Peqt->tncomp;       break;  }
  case planeelementrotlt:{  tncomp=Perlt->tncomp;      break;  }
  case planeelementlq:{     tncomp=Pelq->tncomp;       break;  }
  case planeelementqq:{     tncomp=Peqq->tncomp;       break;  }
  case planeelementrotlq:{  tncomp=Perlq->tncomp;      break;  }
  case planeelementsubqt:{  tncomp=Pesqt->tncomp;      break;  }
  case planequadcontact:{   tncomp=Pqcon->tncomp;      break;  }
  case cctel:{              tncomp=Cct->tncomp;        break;  }
  case dktel:{              tncomp=Dkt->tncomp;        break;  }
  case dstel:{              tncomp=Dst->tncomp;        break;  }
  case q4plateel:{          tncomp=Q4pl->tncomp;       break;  }
  case subsoilplatetr:{     tncomp=Spltr->tncomp;      break;  }
  case subsoilplateq:{      tncomp=Splq->tncomp;       break;  }
  case axisymmlt:{          tncomp=Asymlt->tncomp;     break;  }
  case axisymmlq:{          tncomp=Asymlq->tncomp;     break;  }
  case axisymmqq:{          tncomp=Asymqq->tncomp;     break;  }
  case shelltrelem:{        tncomp=Shtr->tncomp;       break;  }
  case shellqelem:{         tncomp=Shq->tncomp;        break;  }
  case lineartet:{          tncomp=Ltet->tncomp;       break;  }
  case quadrtet:{           tncomp=Qtet->tncomp;       break;  }
  case linearhex:{          tncomp=Lhex->tncomp;       break;  }
  case quadrhex:{           tncomp=Qhex->tncomp;       break;  }
  case lineartetrot:{       tncomp=Ltetrot->tncomp;    break;  }
  case linearhexrot:{       tncomp=Lhexrot->tncomp;    break;  }
  case linearwed:{          tncomp=Lwed->tncomp;       break;  }
  case quadrwed:{           tncomp=Qwed->tncomp;       break;  }
  case particleelem:{       tncomp=Pelem->ndofe;       break;  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  return tncomp;
}



/**
  Function returns number of edges of the element.
   
  @param eid - element id
   
  @return The function returns number of edges.

  Created by JK, 30.12.2001
*/
long mechtop::give_ned (long eid)
{
  long ned=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar3d:{              ned=0;            break;  }
  case beam3d:{             ned=0;            break;  }
  case planeelementlt:{     ned=Pelt->ned;    break;  }
  case planeelementqt:{     ned=Peqt->ned;    break;  }
  case planeelementrotlt:{  ned=Perlt->ned;   break;  }
  case planeelementlq:{     ned=Pelq->ned;    break;  }
  case planeelementqq:{     ned=Peqq->ned;    break;  }
  case planeelementrotlq:{  ned=Perlq->ned;   break;  }
  case planeelementsubqt:{  ned=Pesqt->ned;   break;  }
  case cctel:{              ned=Cct->ned;     break;  }
  case dktel:{              ned=Dkt->ned;     break;  }
  case dstel:{              ned=Dst->ned;     break;  }
  case q4plateel:{          ned=Q4pl->ned;    break;  }
  case subsoilplatetr:{     ned=Spltr->ned;   break;  }
  case subsoilplateq:{      ned=Splq->ned;    break;  }
  case axisymmlt:{          ned=Asymlt->ned;  break;  }
  case axisymmlq:{          ned=Asymlq->ned;  break;  }
  case axisymmqq:{          ned=Asymqq->ned;  break;  }
  case shelltrelem:{        ned=Shtr->ned;    break;  }
  case shellqelem:{         ned=Shq->ned;     break;  }
  case lineartet:{          ned=Ltet->ned;    break;  }
  case quadrtet:{           ned=Qtet->ned;    break;  }
  case linearhex:{          ned=Lhex->ned;    break;  }
  case quadrhex:{           ned=Qhex->ned;    break;  }
  case lineartetrot:{       ned=Ltetrot->ned; break;  }
  case linearhexrot:{       ned=Lhexrot->ned; break;  }
  case linearwed:{          ned=Lwed->ned;    break;  }
  case quadrwed:{           ned=Qwed->ned;    break;  }
  case particleelem:{                         break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return ned;
}



/**
  Function returns number of nodes on one element edge.
   
  @param eid - number of element
   
  @return The function returns number of nodes on element edge

  Created by JK, 30.12.2001
*/
long mechtop::give_nned (long eid)
{
  long nned=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar3d:{              nned=0;             break;  }
  case beam3d:{             nned=0;             break;  }
  case planeelementlt:{     nned=Pelt->nned;    break;  }
  case planeelementqt:{     nned=Peqt->nned;    break;  }
  case planeelementrotlt:{  nned=Perlt->nned;   break;  }
  case planeelementlq:{     nned=Pelq->nned;    break;  }
  case planeelementqq:{     nned=Peqq->nned;    break;  }
  case planeelementrotlq:{  nned=Perlq->nned;   break;  }
  case planeelementsubqt:{  nned=Pesqt->nned;   break;  }
  case cctel:{              nned=Cct->nned;     break;  }
  case dktel:{              nned=Dkt->nned;     break;  }
  case dstel:{              nned=Dst->nned;     break;  }
  case q4plateel:{          nned=Q4pl->nned;    break;  }
  case subsoilplatetr:{     nned=Spltr->nned;   break;  }
  case subsoilplateq:{      nned=Splq->nned;    break;  }
  case axisymmlt:{          nned=Asymlt->nned;  break;  }
  case axisymmlq:{          nned=Asymlq->nned;  break;  }
  case axisymmqq:{          nned=Asymqq->nned;  break;  }
  case shelltrelem:{        nned=Shtr->nned;    break;  }
  case shellqelem:{         nned=Shq->nned;     break;  }
  case lineartet:{          nned=Ltet->nned;    break;  }
  case quadrtet:{           nned=Qtet->nned;    break;  }
  case linearhex:{          nned=Lhex->nned;    break;  }
  case quadrhex:{           nned=Qhex->nned;    break;  }
  case lineartetrot:{       nned=Ltetrot->nned; break;  }
  case linearhexrot:{       nned=Lhexrot->nned; break;  }
  case linearwed:{          nned=Lwed->nned;    break;  }
  case quadrwed:{           nned=Qwed->nned;    break;  }
  case particleelem:{                           break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nned;
}



/**
  Function returns number of surfaces of the element.
   
  @param eid - element id
   
  @return The function returns number of element surfaces.

  Created by JK, 29.4.2004
*/
long mechtop::give_nsurf (long eid)
{
  long nsurf;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case lineartet:{          nsurf=Ltet->nsurf;    break;  }
  case quadrtet:{           nsurf=Qtet->nsurf;    break;  }
  case linearhex:{          nsurf=Lhex->nsurf;    break;  }
  case quadrhex:{           nsurf=Qhex->nsurf;    break;  }
  case lineartetrot:{       nsurf=Ltetrot->nsurf; break;  }
  case linearhexrot:{       nsurf=Lhexrot->nsurf; break;  }
  case linearwed:{          nsurf=Lwed->nsurf;    break;  }
  case quadrwed:{           nsurf=Qwed->nsurf;    break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nsurf;
}



/**
  Function returns number of surfaces of the element.
   
  @param eid - element id
   
  @return The function returns number of nodes on element surfaces

  Created by JK, 29.4.2004
*/
long mechtop::give_nnsurf (long eid)
{
  long nnsurf;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case lineartet:{          nnsurf=Ltet->nnsurf;     break;  }
  case quadrtet:{           nnsurf=Qtet->nnsurf;     break;  }
  case linearhex:{          nnsurf=Lhex->nnsurf;     break;  }
  case quadrhex:{           nnsurf=Qhex->nnsurf;     break;  }
  case lineartetrot:{       nnsurf=Ltetrot->nnsurf;  break;  }
  case linearhexrot:{       nnsurf=Lhexrot->nnsurf;  break;  }
  case linearwed:{          nnsurf=Lwed->nnsurf;     break;  }
  case quadrwed:{           nnsurf=Qwed->nnsurf;     break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nnsurf;
}



/**
  Function returns number of approximated functions.
   
  @param eid - number of element

  @return The function returns number of approximation functions.
   
  Created by JK, 30.12.2001
*/
long mechtop::give_napfun (long eid)
{
  long napfun=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:{              napfun=Bar2d->napfun;      break;  }
  case bar3d:{              napfun=Bar3d->napfun;      break;  }
  case barq2d:{             napfun=Barq2d->napfun;     break;  }
  case barq3d:{             napfun=Barq3d->napfun;     break;  }
  case beam2d:{             napfun=Beam2d->napfun;     break;  }
  case beam3d:{             napfun=Beam3d->napfun;     break;  }
  case beamg3d:{            napfun=Beam3dg->napfun;    break;  }
  case subsoilbeam:{        napfun=Sbeam->napfun;      break;  }
  case beam2dsp:{           napfun=Spbeam2d->napfun;   break;  }
  case spring_1:{           napfun=Spring->napfun;     break;  }
  case spring_2:{           napfun=Spring->napfun;     break;  }
  case spring_3:{           napfun=Spring->napfun;     break;  }
  case spring_4:{           napfun=Spring->napfun;     break;  }
  case spring_5:{           napfun=Spring->napfun;     break;  }
  case spring_6:{           napfun=Spring->napfun;     break;  }
  case planeelementlt:{     napfun=Pelt->napfun;       break;  }
  case planeelementqt:{     napfun=Peqt->napfun;       break;  }
  case planeelementrotlt:{  napfun=Perlt->napfun;      break;  }
  case planeelementlq:{     napfun=Pelq->napfun;       break;  }
  case planeelementqq:{     napfun=Peqq->napfun;       break;  }
  case planeelementrotlq:{  napfun=Perlq->napfun;      break;  }
  case planeelementsubqt:{  napfun=Pesqt->napfun;      break;  }
  case cctel:{              napfun=Cct->napfun;        break;  }
  case dktel:{              napfun=Dkt->napfun;        break;  }
  case dstel:{              napfun=Dst->napfun;        break;  }
  case q4plateel:{          napfun=Q4pl->napfun;       break;  }
  case subsoilplatetr:{     napfun=Spltr->napfun;      break;  }
  case subsoilplateq:{      napfun=Splq->napfun;       break;  }
  case axisymmlt:{          napfun=Asymlt->napfun;     break;  }
  case axisymmlq:{          napfun=Asymlq->napfun;     break;  }
  case axisymmqq:{          napfun=Asymqq->napfun;     break;  }
  case shelltrelem:{        napfun=Shtr->napfun;       break;  }
  case shellqelem:{         napfun=Shq->napfun;        break;  }
  case lineartet:{          napfun=Ltet->napfun;       break;  }
  case quadrtet:{           napfun=Qtet->napfun;       break;  }
  case linearhex:{          napfun=Lhex->napfun;       break;  }
  case quadrhex:{           napfun=Qhex->napfun;       break;  }
  case lineartetrot:{       napfun=Ltetrot->napfun;    break;  }
  case linearhexrot:{       napfun=Lhexrot->napfun;    break;  }
  case linearwed:{          napfun=Lwed->napfun;       break;  }
  case quadrwed:{           napfun=Qwed->napfun;       break;  }
  case particleelem:{                                  break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return napfun;
}



/**
  Function returns dimension of problem.
   
  @param eid - number of element
   
  @retval 1 - for 1D
  @retval 2 - for 2D
  @retval 3 - for 3D

  Created by JK, 23.2.2002
*/
long mechtop::give_dimension (long eid)
{
  long dim=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:{              dim=1;  break;  }
  case bar3d:{              dim=1;  break;  }
  case barq2d:{             dim=1;  break;  }
  case barq3d:{             dim=1;  break;  }
  case beam2d:{             dim=1;  break;  }
  case beam3d:{             dim=1;  break;  }
  case beamg3d:{            dim=1;  break;  }
  case subsoilbeam:{        dim=1;  break;  }
  case beam2dsp:{           dim=1;  break;  }
  case spring_1:{           dim=1;  break;  }
  case spring_2:{           dim=1;  break;  }
  case spring_3:{           dim=1;  break;  }
  case spring_4:{           dim=1;  break;  }
  case spring_5:{           dim=1;  break;  }
  case spring_6:{           dim=1;  break;  }
  case planeelementlt:{     dim=2;  break;  }
  case planeelementqt:{     dim=2;  break;  }
  case planeelementrotlt:{  dim=2;  break;  }
  case planeelementlq:{     dim=2;  break;  }
  case planeelementqq:{     dim=2;  break;  }
  case planeelementrotlq:{  dim=2;  break;  }
  case planeelementsubqt:{  dim=2;  break;  }
  case cctel:{              dim=2;  break;  }
  case dktel:{              dim=2;  break;  }
  case dstel:{              dim=2;  break;  }
  case q4plateel:{          dim=2;  break;  }
  case axisymmlt:{          dim=2;  break;  }
  case axisymmlq:{          dim=2;  break;  }
  case axisymmqq:{          dim=2;  break;  }
  case subsoilplatetr:{     dim=2;  break;  }
  case subsoilplateq:{      dim=2;  break;  }
  case shelltrelem:{        dim=2;  break;  }
  case shellqelem:{         dim=2;  break;  }
  case lineartet:{          dim=3;  break;  }
  case quadrtet:{           dim=3;  break;  }
  case linearhex:{          dim=3;  break;  }
  case quadrhex:{           dim=3;  break;  }
  case lineartetrot:{       dim=3;  break;  }
  case linearhexrot:{       dim=3;  break;  }
  case linearwed:{          dim=3;  break;  }
  case quadrwed:{           dim=3;  break;  }
  case particleelem:{               break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return dim;
}



/**
  Function returns number of blocks in characteristic matrices.
   
  @param eid - element id
   
  @return The function returns number of blocks.

  Created by JK, 8.5.2002
*/
long mechtop::give_nb (long eid)
{
  long nb=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){

  case bar2d:{              nb=Bar2d->nb;      break;  }
  case bar3d:{              nb=Bar3d->nb;      break;  }
  case barq2d:{             nb=Barq2d->nb;     break;  }
  case barq3d:{             nb=Barq3d->nb;     break;  }
  case beam2d:{             nb=Beam2d->nb;     break;  }
  case beam3d:{             nb=Beam3d->nb;     break;  }
  case beamg3d:{            nb=Beam3dg->nb;    break;  }
  case subsoilbeam:{        nb=Sbeam->nb;      break;  }
  case beam2dsp:{           nb=Spbeam2d->nb;   break;  }
  case spring_1:{           nb=Spring->nb;     break;  }
  case spring_2:{           nb=Spring->nb;     break;  }
  case spring_3:{           nb=Spring->nb;     break;  }
  case spring_4:{           nb=Spring->nb;     break;  }
  case spring_5:{           nb=Spring->nb;     break;  }
  case spring_6:{           nb=Spring->nb;     break;  }
  case planeelementlt:{     nb=Pelt->nb;       break;  }
  case planeelementqt:{     nb=Peqt->nb;       break;  }
  case planeelementrotlt:{  nb=Perlt->nb;      break;  }
  case planeelementlq:{     nb=Pelq->nb;       break;  }
  case planeelementqq:{     nb=Peqq->nb;       break;  }
  case planeelementrotlq:{  nb=Perlq->nb;      break;  }
  case planeelementsubqt:{  nb=Pesqt->nb;      break;  }
  case planequadcontact:{   nb=Pqcon->nb;      break;  }
  case cctel:{              nb=Cct->nb;        break;  }
  case dktel:{              nb=Dkt->nb;        break;  }
  case dstel:{              nb=Dst->nb;        break;  }
  case q4plateel:{          nb=Q4pl->nb;       break;  }
  case subsoilplatetr:{     nb=Spltr->nb;      break;  }
  case subsoilplateq:{      nb=Splq->nb;       break;  }
  case axisymmlt:{          nb=Asymlt->nb;     break;  }
  case axisymmlq:{          nb=Asymlq->nb;     break;  }
  case axisymmqq:{          nb=Asymqq->nb;     break;  }
  case shelltrelem:{        nb=Shtr->nb;       break;  }
  case shellqelem:{         nb=Shq->nb;        break;  }
  case lineartet:{          nb=Ltet->nb;       break;  }
  case quadrtet:{           nb=Qtet->nb;       break;  }
  case linearhex:{          nb=Lhex->nb;       break;  }
  case quadrhex:{           nb=Qhex->nb;       break;  }
  case lineartetrot:{       nb=Ltetrot->nb;    break;  }
  case linearhexrot:{       nb=Lhexrot->nb;    break;  }
  case linearwed:{          nb=Lwed->nb;       break;  }
  case quadrwed:{           nb=Qwed->nb;       break;  }
  case particleelem:{       nb=Pelem->nb;      break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return nb;
}



/**
  Function returns number of layeres of layered element.
   
  @param eid - element id
 
  @return The function returns number of layers,
  
  Created by JK, 7.12.2002
*/
/*
long mechtop::give_nl (long eid)
{
  return Gtm->give_nl (eid);
}
*/



/**
  The function returns stress/strain state indicator.
  
  @param eid - element id
  @param bi - block id

  @retval bar - for 1D stress state
  @retval plbeam - for 2D beam stress state
  @retval spacebeam - for 3D beam stress state
  @retval planestress - for 2D plane stress state
  @retval planestrain - for 2D plane strain state
  @retval platek      - plate stress state
  @retval plates      - plate stress state
  @retval axisymm     - for 2D axisymmetric stress state
  @retval shell       - for 3D shell stress state
  @retval spacestress - for fully 3D space stress state
  
  Created by JK, 7.12.2002
*/
strastrestate mechtop::give_ssst (long eid,long bi)
{
  strastrestate ssst;
  elemtype te;
  
  te = give_elem_type (eid);
  switch (te){
  case bar2d:{              ssst = Bar2d->ssst;         break;  }
  case bar3d:{              ssst = Bar3d->ssst;         break;  }
  case barq2d:{             ssst = Barq2d->ssst;        break;  }
  case barq3d:{             ssst = Barq3d->ssst;        break;  }
  case beam2d:{             ssst = Beam2d->ssst;        break;  }
  case beam3d:{             ssst = Beam3d->ssst;        break;  }
  case beamg3d:{            ssst = Beam3dg->ssst;       break;  }
  case subsoilbeam:{        ssst = Sbeam->ssst;         break;  }
  case beam2dsp:{           ssst = Spbeam2d->ssst;      break;  }
  case spring_1:{           ssst = Spring->ssst;        break;  }
  case spring_2:{           ssst = Spring->ssst;        break;  }
  case spring_3:{           ssst = Spring->ssst;        break;  }
  case spring_4:{           ssst = Spring->ssst;        break;  }
  case spring_5:{           ssst = Spring->ssst;        break;  }
  case spring_6:{           ssst = Spring->ssst;        break;  }
  case planeelementlt:{     ssst = elements[eid].ssst;  break;  }
  case planeelementqt:{     ssst = elements[eid].ssst;  break;  }
  case planeelementrotlt:{  ssst = elements[eid].ssst;  break;  }
  case planeelementlq:{     ssst = elements[eid].ssst;  break;  }
  case planeelementqq:{     ssst = elements[eid].ssst;  break;  }
  case planeelementrotlq:{  ssst = elements[eid].ssst;  break;  }
  case planeelementsubqt:{  ssst = elements[eid].ssst;  break;  }
  case cctel:{              ssst = Cct->ssst;           break;  }
  case dktel:{              ssst = Dkt->ssst;           break;  }
  case dstel:{              ssst = Dst->ssst;           break;  }
  case q4plateel:{          ssst = Q4pl->ssst;          break;  }
  case subsoilplatetr:{     ssst = Spltr->ssst;         break;  }
  case subsoilplateq:{      ssst = Splq->ssst;          break;  }
  case axisymmlt:{          ssst = Asymlt->ssst;        break;  }
  case axisymmlq:{          ssst = Asymlq->ssst;        break;  }
  case axisymmqq:{          ssst = Asymqq->ssst;        break;  }
  case shelltrelem:{
    if (bi==0 || bi==1)  ssst = (strastrestate) planestress;
    if (bi==2)           ssst = (strastrestate) platek;
    if (bi==3)           ssst = (strastrestate) shell;
    break;  }
  case shellqelem:{
    if (bi==0 || bi==1)  ssst = (strastrestate) planestress;
    if (bi==2 || bi==3)  ssst = (strastrestate) plates;
    if (bi==4)           ssst = (strastrestate) shell;
    break;  }
  case lineartet:{          ssst = Ltet->ssst;          break;  }
  case quadrtet:{           ssst = Qtet->ssst;          break;  }
  case linearhex:{          ssst = Lhex->ssst;          break;  }
  case quadrhex:{           ssst = Qhex->ssst;          break;  }
  case lineartetrot:{       ssst = Ltetrot->ssst;       break;  }
  case linearhexrot:{       ssst = Lhexrot->ssst;       break;  }
  case linearwed:{          ssst = Lwed->ssst;          break;  }
  case quadrwed:{           ssst = Qwed->ssst;          break;  }
  case particleelem:{                                   break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return ssst;
}



/**
  Function returns degree of polynomial expansion.
   
  @param eid - number of element
   
  @return The function returns degree of appproximation. 
  
  Created by JK,
*/
long mechtop::give_degree (long eid)
{
  long deg=0;
  elemtype te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case bar2d:{                deg=1;  break;  }
  case bar3d:{                deg=1;  break;  }
  case barq2d:{               deg=2;  break;  }
  case barq3d:{               deg=2;  break;  }
  case beam2d:{               deg=1;  break;  }
  case beam3d:{               deg=1;  break;  }
  case beamg3d:{              deg=3;  break;  }
  case spring_1:{             deg=1;  break;  }
  case spring_2:{             deg=1;  break;  }
  case spring_3:{             deg=1;  break;  }
  case spring_4:{             deg=1;  break;  }
  case spring_5:{             deg=1;  break;  }
  case spring_6:{             deg=1;  break;  }
  case subsoilbeam:{          deg=1;  break;  }
  case beam2dsp:{             deg=1;  break;  }
  case planeelementlt:{       deg=1;  break;  }
  case planeelementqt:{       deg=2;  break;  }
  case planeelementrotlt:{    deg=1;  break;  }
  case planeelementlq:{       deg=1;  break;  }
  case planeelementqq:{       deg=2;  break;  }
  case planeelementrotlq:{    deg=1;  break;  }
  case planeelementsubqt:{    deg=2;  break;  }
  case cctel:{                deg=1;  break;  }
  case dktel:{                deg=1;  break;  }
  case dstel:{                deg=1;  break;  }
  case q4plateel:{            deg=1;  break;  }
  case subsoilplatetr:{       deg=1;  break;  }
  case subsoilplateq:{        deg=1;  break;  }
  case axisymmlt:{            deg=1;  break;  }
  case axisymmlq:{            deg=1;  break;  }
  case axisymmqq:{            deg=1;  break;  }
  case shelltrelem:{          deg=1;  break;  }
  case shellqelem:{           deg=1;  break;  }
  case lineartet:{            deg=1;  break;  }
  case quadrtet:{             deg=2;  break;  }
  case linearhex:{            deg=1;  break;  }
  case quadrhex:{             deg=2;  break;  }
  case lineartetrot:{         deg=1;  break;  }
  case linearhexrot:{         deg=1;  break;  }
  case linearwed:{            deg=1;  break;  }
  case quadrwed:{             deg=2;  break;  }
  case particleelem:{                 break;  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  return deg;
}



/**
  Function returns node numbers on required element edge.
   
  @param eid - element id
  @param edid - edge id
  @param nodes - array containing edge nodes (output)
   
  @return The function returns edge nodes in the parameter nodes.

  Created by JK, 5.3.2002
*/
void mechtop::give_edge_nodes (long eid,long edid,long *nodes)
{
  long nne;
  elemtype te;
  ivector nod;
  
  te = give_elem_type (eid);
  nne = give_nne (eid);
  allocv (nne,nod);
  give_elemnodes (eid,nod);
  
  switch (te){
  case planeelementlt:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[2]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[0]; }
    break;
  }
  case planeelementqt:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[1];  nodes[2]=nod[3]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[2];  nodes[2]=nod[4]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[0];  nodes[2]=nod[5]; }
    break;
  }
  case planeelementrotlt:{
    if (edid==0){  nodes[0]=nod[1];  nodes[1]=nod[2]; }
    if (edid==1){  nodes[0]=nod[2];  nodes[1]=nod[0]; }
    if (edid==2){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    break;
  }
  case planeelementlq:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[2]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[3]; }
    if (edid==3){  nodes[0]=nod[3];  nodes[1]=nod[0]; }
    break;
  }
  case planeelementqq:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[4];  nodes[2]=nod[1]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[5];  nodes[2]=nod[2]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[6];  nodes[2]=nod[3]; }
    if (edid==3){  nodes[0]=nod[3];  nodes[1]=nod[7];  nodes[2]=nod[0]; }
    break;
  }
  case planeelementrotlq:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[2]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[3]; }
    if (edid==3){  nodes[0]=nod[3];  nodes[1]=nod[0]; }
    break;
  }
  case planeelementsubqt:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[1];  nodes[2]=nod[3]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[2];  nodes[2]=nod[4]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[0];  nodes[2]=nod[5]; }
    break;
  }
  case axisymmlt:{
    if (edid==0){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==1){  nodes[0]=nod[1];  nodes[1]=nod[2]; }
    if (edid==2){  nodes[0]=nod[2];  nodes[1]=nod[0]; }
    break;
  }
  case linearhex:{
    if (edid==0){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==1){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==2){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==3){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==4){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==5){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==6){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==7){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==8){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==9){   nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==10){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    if (edid==11){  nodes[0]=nod[0];  nodes[1]=nod[1]; }
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  destrv (nod);
}



/**
  Function returns numbers of nodes defining apropriate element.
   
  @param eid - element id
  @param nodes - array containing nodes (output)

  @return The function returns node numbers in the parameter nodes.
   
  Created by JK, 22.7.2001
*/
void mechtop::give_elemnodes (long eid,ivector &nodes)
{
  Gtm->give_nodes (eid,nodes);
}



/**
  Function extracts all code numbers of actual element.
   
  @param eid - element id
  @param cn - array containing all code numbers on element (output)
   
  @return The function returns code numbers in the parameter cn.

  Created by JK, 25.6.2001
*/
void mechtop::give_code_numbers (long eid,long *cn)
{
  Gtm->give_code_numbers (eid,cn);
}



/**
  Function extracts code numbers of actual node
   
  @param nid - node id
  @param cn - array containing code numbers on node (output)
   
  @return The function returns code numbers in the parameter cn.

  Created by JK, 7.12.2002
*/
void mechtop::give_node_code_numbers (long nid,long *cn)
{
  Gtm->give_node_code_numbers (nid,cn);
}



/**
  Function extracts code numbers of multipliers on actual node
   
  @param nid - node id
  @param lid - layer id
  @param cn - array containing all code numbers on node (output)
   
  @return The function returns code numbers of multipliers in the parameter cn.

  Created by JK, 7.12.2002
*/
void mechtop::give_mult_code_numbers (long nid,long lid,long *cn)
{
  Gtm->give_mult_code_numbers (nid,lid,cn);
}



/**
  Function returns node coordinates of the appropriate element.
   
  @param x - vector containing node coordinates (output)
  @param eid - element id
   
  @return The function returns x-coordinates in the parameter x. 

  Created by JK, 19.7.2001
*/
void mechtop::give_node_coord1d (vector &x,long eid)
{
  Gtm->give_node_coord1d (x,eid);
}



/**
  Function returns node coordinates of the appropriate element.
   
  @param x,y - vectors containing node coordinates (output)
  @param eid - element id
   
  @return The function returns 2D(x,y) coordinates in parameters x and y. 

  Created by JK, 19.7.2001
*/
void mechtop::give_node_coord2d (vector &x,vector &y,long eid)
{
  Gtm->give_node_coord2d (x,y,eid);
}



/**
  Function returns node coordinates of the appropriate element.
   
  @param x,z - vectors containing coordinates
  @param eid - element id
   
  @return The function returns 2D(x,z) coordinates in parameters x and z. 

  Created by JK, 19.7.2001
*/
void mechtop::give_node_coord2dxz (vector &x,vector &z,long eid)
{
  Gtm->give_node_coord2dxz (x,z,eid);
}



/**
  Function returns node coordinates of the appropriate element.
   
  @param x,y,z - vectors containing coordinates (output)
  @param eid - element id
   
  @return The function returns 3D(x,y,z) coordinates in parameters x, y and z. 

  Created by JK, 19.7.2001
*/
void mechtop::give_node_coord3d (vector &x,vector &y,vector &z,long eid)
{
  Gtm->give_node_coord3d (x,y,z,eid);
}



/**
  Function returns length of the appropriate element.

  @param eid - element id

  @return The function returns element length.

  Created by JK, 19.7.2001
*/
double mechtop::give_length (long eid)
{
  long nn = give_nne(eid);
  double l;
  if (nn > 2)
    print_err("invalid number of nodes on element is detected", __FILE__, __LINE__, __func__);

  vector x(nn), y(nn), z(nn);
  Gtm->give_node_coord3d (x, y, z, eid);
  l = sqrt(sqr(x[1]-x[0]) + sqr(y[1]-y[0]) + sqr(z[1]-z[0]));
  return l;
}



/**
  Function returns area of the appropriate element.

  @param eid - element id

  @return The function returns area of 2D element.

  Created by JK, 19.7.2001
*/
double mechtop::give_area (long eid)
{
  long nn = give_nne(eid);
  double a = 0.0;
  if (nn < 2)
    print_err("invalid number of nodes on element is detected", __FILE__, __LINE__, __func__);

  vector x(nn), y(nn), z(nn);
  Gtm->give_node_coord3d (x, y, z, eid);
  switch (Mt->give_elem_type(eid)){
  case planeelementlt:
  case planeelementqt:
  case planeelementrotlt:
  case axisymmlt:
    a = ((x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]))/2.0;
    break;
  case planeelementlq:
  case planeelementqq:
  case planeelementrotlq:
  case axisymmlq:
    a = ((x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]) + (x[2]-x[0])*(y[3]-y[0])-(x[3]-x[0])*(y[2]-y[0]) ) / 2.0;
    break;
  case cctel:
  case dktel:
  case dstel:
    a = ((x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]))/2.0;
    break;
  case q4plateel:
    a = ((x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]) + (x[2]-x[0])*(y[3]-y[0])-(x[3]-x[0])*(y[2]-y[0]) ) / 2.0;
    break;
  default:
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  return a;
}



/**
  Function returns volume of the appropriate element.

  @param eid - element id

  @return The function returns area of 3D element.

  Created by JK, 19.7.2001
*/
double mechtop::give_volume (long eid)
{
  long nn = give_nne(eid);
  double v = 0.0;
  if (nn < 2)
    print_err("invalid number of nodes on element is detected", __FILE__, __LINE__, __func__);

  vector x(nn), y(nn), z(nn);
  Gtm->give_node_coord3d (x, y, z, eid);
  switch (Mt->give_elem_type(eid)){
  case lineartet:
  case quadrtet:
    jac_3d (v, x, y, z, 0.0, 0.0, 0.0);
    v/=6.0;
    break;
  case linearhex:
  case quadrhex:
  case lineartetrot:
  case linearhexrot:
    jac_3d (v, x, y, z, 1.0, 1.0, 1.0);
    v*=8.0;
    break;
  default:
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  return v;
}



/**
  Function returns maximum number of components of strain/stress array at nodes in the 
  parameter max_ncompstrn and maximum number of components of strain/stress array on 
  elements in the parameter max_ncompstre
   
  @param max_ncompstrn - maximum number of components of strain/stress array at nodes (output)
  @param max_ncompstre - maximum number of components of strain/stress array on elements (output)

  @return The function returns maximum number stress/strain components in the parameters max_ncompstrn 
          and max_ncompstre.
   
  Created by Tomas Koudelka,
*/
void mechtop::give_maxncompstr (long &max_ncompstrn, long &max_ncompstre)
{
  long i;
  max_ncompstrn = max_ncompstre = 0;

  for (i=0;i<ne;i++)
  {
    if (max_ncompstre < Mm->ip[elements[i].ipp[0][0]].ncompstr)
      max_ncompstre = Mm->ip[elements[i].ipp[0][0]].ncompstr;
  }      
  for (i=0;i<nn;i++)
  {
    if (max_ncompstrn < nodes[i].ncompstr)
      max_ncompstrn = nodes[i].ncompstr;
  }      
}



/**
  Function returns maximum number of components of other array at nodes in the  
  parameter max_nncompo and maximum number of components of eqother array on 
  elements in the parameter max_encompo
   
  @param max_encompo - maximum number of components of other array at nodes
  @param max_nncompo - maximum number of components of eqother array on elements

  @return The function returns maximum number other array components in the parameters max_nncompo 
          and max_encompo.
   
  Created by Tomas Koudelka,
*/
void mechtop::give_maxncompo (long &max_nncompo, long &max_encompo)
{
  long i;
  max_nncompo = max_encompo = 0;

  for (i=0;i<ne;i++)
  {
    if (max_encompo < Mm->ip[elements[i].ipp[0][0]].ncompeqother)
      max_encompo = Mm->ip[elements[i].ipp[0][0]].ncompeqother;
  }      
  for (i=0;i<nn;i++)
  {
    if (max_nncompo < nodes[i].ncompother)
      max_nncompo = nodes[i].ncompother;
  }      
}



/**
  Function returns numbers of local coordinate systems at nodes.
   
  @param nod - array containing node numbers

  @return The function returns id of local coordinate system in the parameter nod.

  Created by JK,
*/
long mechtop::locsystems (ivector &nod)
{
  long i,j,n;
  n=nod.n;
  j=0;
  for (i=0;i<n;i++){
    j+=nodes[nod[i]].transf;
  }
  return j;
}



/**
  Function alocates array containing reactions.

  @return The function does not return anything.

  Created by JK,
*/
void mechtop::comreacnod (void)
{
  long i,j,ndofn;
  
  for (i=0;i<nn;i++){
    ndofn = Gtm->give_ndofn (i);
    for (j=0;j<ndofn;j++){
      if (Gtm->give_dof (i,j)<1){
	nodes[i].react=1;
	if (nodes[i].r==NULL)
	  nodes[i].r = new double [ndofn];
	memset (nodes[i].r,0,ndofn*sizeof(double));
	break;
      }
    }
  }
}



/**
  Function marks elements necessary for reaction evaluation.

  @return The function does not return anything.

  Created by JK,
*/
void mechtop::comreacelem (void)
{
  long i,j,k,nne;
  ivector nod;
  
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      nne=Gtm->give_nne (i);
      allocv (nne,nod);
      Gtm->give_nodes (i,nod);
      for (j=0;j<nne;j++){
	k=nod[j];
	if (nodes[k].react==1){
	  elements[i].react=1;
	  break;
	}
      }
      destrv (nod);
    }
  }
}



/**
  Function marks elements necessary for reaction evaluation and
  alocates array containing reactions.

  @return The function does not return anything.

  Created by JK,
*/
void mechtop::comreac (void)
{
  comreacnod ();
  comreacelem ();
}



/**
  Function computes nodal values of strains with help of least square problem solution.
   
  @param nodes - array containing nodes of element
  @param nx,ny,nz - arrays containing natural coordinates of nodes
  @param lhs - array of coefficients of linear functions (solution of least square problem)
  @param dim - problem dimension
  @param fi - position of first component in nodal strain array
  @param ncomp - number of stored components
  @param lcid - load case id

  @return The function does not return anything.

  Created by JK, 10.5.2002
*/
void mechtop::strain_nodal_values (ivector &nod,vector &nx,vector &ny,vector &nz,
				   double *lhs,long dim,long fi,long ncomp,long lcid)
{
  long i,j;
  vector eps;
  
  allocv (ncomp,eps);

  if (dim==1){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	eps[j] = lhs[j*2]+lhs[j*2+1]*nx[i];
      }
      nodes[nod[i]].storestrain (lcid,fi,ncomp,eps);
    }
  }
  if (dim==2){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	eps[j] = lhs[j*3]+lhs[j*3+1]*nx[i]+lhs[j*3+2]*ny[i];
      }
      nodes[nod[i]].storestrain (lcid,fi,ncomp,eps);
    }
  }
  if (dim==3){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	eps[j] = lhs[j*4]+lhs[j*4+1]*nx[i]+lhs[j*4+2]*ny[i]+lhs[j*4+3]*nz[i];
      }
      nodes[nod[i]].storestrain (lcid,fi,ncomp,eps);
    }
  }
  
  destrv (eps);
}



/**
  Function computes nodal values of stresses with help of least square problem solution.
   
  @param nodes - array containing nodes of element
  @param nx,ny,nz - arrays containing natural coordinates of nodes
  @param lhs - array of coefficients of linear functions (solution of least square problem)
  @param dim - problem dimension
  @param fi - position of first component in nodal strain array
  @param ncomp - number of stored components
  @param lcid - load case id

  @return The function does not return anything.

  Created by JK, 10.5.2002
*/
void mechtop::stress_nodal_values (ivector &nod,vector &nx,vector &ny,vector &nz,
				   double *lhs,long dim,long fi,long ncomp,long lcid)
{
  long i,j;
  vector sig;
  
  allocv (ncomp,sig);
  if (dim==2){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	//Mt->nodes[nodes[i]].stress[pos+j] = lhs[j*3]+lhs[j*3+1]*nx[i]+lhs[j*3+2]*ny[i];
	sig[j] = lhs[j*3]+lhs[j*3+1]*nx[i]+lhs[j*3+2]*ny[i];
      }
      nodes[nod[i]].storestress (lcid,fi,ncomp,sig);

    }
  }
  if (dim==3){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	//Mt->nodes[nodes[i]].stress[pos+j] = lhs[j*4]+lhs[j*4+1]*nx[i]+lhs[j*4+2]*ny[i]+lhs[j*4+3]*nz[i];
	sig[j] = lhs[j*4]+lhs[j*4+1]*nx[i]+lhs[j*4+2]*ny[i]+lhs[j*4+3]*nz[i];
      }
      nodes[nod[i]].storestress (lcid,fi,ncomp,sig);
    }
  }
  destrv (sig);
}



/**
  Function computes nodal values of other quantities with help of least square problem solution.
   
  @param nodes - array containing nodes of element
  @param nx,ny,nz - arrays containing natural coordinates of nodes
  @param lhs - array of coefficients of linear functions (solution of least square problem)
  @param dim - problem dimension
  @param fi - position of first component in nodal strain array
  @param ncomp - number of stored components
  @param lcid - load case id

  @return The function does not return anything.

  Created by JK, 10.5.2002
*/
void mechtop::other_nodal_values (ivector &nod,vector &nx,vector &ny,vector &nz,
				   double *lhs,long dim,long fi,long ncomp,long lcid)
{
  long i,j;
  vector other;
  
  allocv (ncomp,other);
  if (dim==1){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	other[j] = lhs[j*2]+lhs[j*2+1]*nx[i];
      }
      nodes[nod[i]].storeother (lcid,fi,ncomp,other);
    }
  }
  if (dim==2){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	other[j] = lhs[j*3]+lhs[j*3+1]*nx[i]+lhs[j*3+2]*ny[i];
      }
      nodes[nod[i]].storeother (lcid,fi,ncomp,other);

    }
  }
  if (dim==3){
    for (i=0;i<nod.n;i++){
      for (j=0;j<ncomp;j++){
	other[j] = lhs[j*4]+lhs[j*4+1]*nx[i]+lhs[j*4+2]*ny[i]+lhs[j*4+3]*nz[i];
      }
      nodes[nod[i]].storeother (lcid,fi,ncomp,other);
    }
  }
  destrv (other);
}



/**
  The function converts code numbers stored at nodes to code numbers
  stored on elements. It is used for mixed 2D mesh of plane elements,
  bars and springs.

  @return The function does not return anything.

  Created by Tomas Koudelka - koudelka@cml.fsv.cvut.cz
*/
void mechtop::store_code_num_elem(void)
{
  long i, j, nne, ndofe, ndofn;
  elemtype te;
  ivector nod;
  long *cn;
  for(i=0; i < ne; i++)
  {
    te    = give_elem_type(i);
    nne   = give_nne(i);
    ndofe = give_ndofe(i);
    allocv(nne, nod);
    give_elemnodes(i, nod);
    switch(te)
    {
      case beam2d:
        break;
      case spring_1:
      {
        ndofn = give_ndofn(nod[0]);
        if (ndofn < 1)
        {
          print_err("spring_1 is connected to node %ld with wrong number of dofs(%ld)", __FILE__, __LINE__, __func__, nod[0], ndofn);
          abort();
        }
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofn];
          cn = new long[ndofn];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofn);
          Mt->give_node_code_numbers(nod[0], cn);
          Gtm->gelements[i].cn[0] = cn[0];
          delete [] cn; 
        }
        break;
      }
      case spring_2:
      {
        ndofn = give_ndofn(nod[0]);
        if (ndofn < 2)
        {
          print_err("spring_2 is connected to node %ld with wrong number of dofs(%ld)", __FILE__, __LINE__, __func__, nod[0], ndofn);
          abort();
        }
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofn];
          cn = new long[ndofn];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofn);
          Mt->give_node_code_numbers(nod[0], cn);
          Gtm->gelements[i].cn[1] = cn[1];
          delete [] cn; 
        }
        break;
      }
      case spring_3:
      {
        ndofn = give_ndofn(nod[0]);
        if (ndofn < 3)
        {
          print_err("spring_3 is connected to node %ld with wrong number of dofs(%ld)", __FILE__, __LINE__, __func__, nod[0], ndofn);
          abort();
        }
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofn];
          cn = new long[ndofn];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofn);
          Mt->give_node_code_numbers(nod[0], cn);
          Gtm->gelements[i].cn[2] = cn[2];
          delete [] cn; 
        }
        break;
      }
      case spring_4:
      {
        ndofn = give_ndofn(nod[0]);
        if (ndofn < 1)
        {
          print_err("spring_4 is connected to node %ld with wrong number of dofs(%ld)", __FILE__, __LINE__, __func__, nod[0], ndofn);
          abort();
        }
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofn];
          cn = new long[ndofn];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofn);
          Mt->give_node_code_numbers(nod[0], cn);
          Gtm->gelements[i].cn[3] = cn[3];
          delete [] cn; 
        }
        break;
      }
      case spring_5:
      {
        ndofn = give_ndofn(nod[0]);
        if (ndofn < 1)
        {
          print_err("spring_5 is connected to node %ld with wrong number of dofs(%ld)", __FILE__, __LINE__, __func__, nod[0], ndofn);
          abort();
        }
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofn];
          cn = new long[ndofn];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofn);
          Mt->give_node_code_numbers(nod[0], cn);
          Gtm->gelements[i].cn[4] = cn[4];
          delete [] cn; 
        }
        break;
      }
      case spring_6: 
      {
        if (ndofn < 1)
        {
          print_err("spring_6 is connected to node %ld with wrong number of dofs(%ld)", __FILE__, __LINE__, __func__, nod[0], ndofn);
          abort();
        }
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofn];
          cn = new long[ndofn];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofn);
          Mt->give_node_code_numbers(nod[0], cn);
          Gtm->gelements[i].cn[5] = cn[5];
          delete [] cn; 
        }
        break;
      }
      case bar2d:
      case barq2d:
      case planeelementlt:
      case planeelementqt:
      case planeelementlq:
      case planeelementqq:
        if (Gtm->gelements[i].cne == 0)
        {
          Gtm->gelements[i].cne = 1;
          Gtm->gelements[i].cn = new long[ndofe];
          memset(Gtm->gelements[i].cn, 0, sizeof(*Gtm->gelements[i].cn)*ndofe);
          for (j=0; j<nne; j++)
          {
            ndofn = Gtm->give_ndofn(nod[j]);
            cn = new long[ndofn];
            Mt->give_node_code_numbers(nod[j], cn);
            Gtm->gelements[i].cn[2*j]   = cn[0];
            Gtm->gelements[i].cn[2*j+1] = cn[1];
            delete [] cn; 
          }
        }
        break;
      default:
        print_err("unknown type of element is required", __FILE__, __LINE__, __func__); 
    }
  }
}



/**
  Function allocates arrays defined on nodes.
  Arrays strain, stress, other, ncontr_strain, ncontr_stress, ncontr_other are allocated on each node.
   
  @return The function does not return anything.

  Created by JK, 28.11.2006
*/
void mechtop::alloc_nodes_arrays (void)
{
  long i,j,nne,ncomp, ncompo;
  ivector nod;
  strastrestate ssst;
  elemtype te;
  
  if (Mp->strainpos==2 || Mp->strainpos==3 || Mp->stresspos==2 || Mp->stresspos==3 || Mp->otherpos==2){
    for (i=0;i<ne;i++){
      if (Gtm->leso[i]==1){
	//  number of nodes on element
	nne = give_nne (i);
	allocv (nne,nod);
	give_elemnodes (i,nod);
	//  number of strain/stress components
	ncomp=give_tncomp (i);
	//  number of components in array other
	ncompo=Mm->ip[elements[i].ipp[0][0]].ncompeqother;     
	
	//  element type
	te = give_elem_type (i);
	if (te != shelltrelem && te != shellqelem){
	  //  strain/stress state (beam, plane stress, plane strain, etc.
	  ssst=Mt->give_ssst (i,0);
	  if ((ssst == planestrain) || (ssst == planestress)){
	    ncomp=4;
	  }
	}
	
	
	for (j=0;j<nne;j++){
	  if (nodes[nod[j]].strain==NULL){
	    //  the main loop is over elements because ncomp is defined on elements
	    //  this condition skips already allocated nodes
	    //  nodes are called several times
	    nodes[nod[j]].alloc_strain (ncomp,Mb->nlc);
	  }
	  if (nodes[nod[j]].stress==NULL){
	    //  the main loop is over elements because ncomp is defined on elements
	    //  this condition skips already allocated nodes
	    //  nodes are called several times
	    nodes[nod[j]].alloc_stress (ncomp,Mb->nlc);
	  }
	  if (nodes[nod[j]].other==NULL){
	    //  the main loop is over elements because ncomp is defined on elements
	    //  this condition skips already allocated nodes
	    //  nodes are called several times
	    nodes[nod[j]].alloc_other (ncompo,Mb->nlc);
	  }
	  if (ncompo > nodes[nod[j]].ncompother){
	    //  if different number of components are used, the maximum number has to be used
	    nodes[nod[j]].realloc(ncompo,Mb->nlc);
	  }
 	}
	destrv (nod);
      }
    }
  }
}



/**
  Function allocates end nodes.
  End nodes are allocated in problems with hemivariational inequalities.
   
  @return The function does not return anything.

  Created by JK, 1.3.2009
*/
void mechtop::alloc_enodes ()
{
  if (Gtm->nen>0){
    enodes = new endnodem [Gtm->nen];
  }
}



/**
  Functions allocates edges.
  Edges are allocated in problems with hemivariational inequalities.
   
  @return The function does not return anything.

  Created by JK, 1.3.2009
*/
void mechtop::alloc_edges ()
{
  if (Gtm->nged>0){
    edges = new edgem [Gtm->nged];
  }
}



/**
  Function allocates nodes and elements for preprocessor

  @param nn - number of nodes
  @param ne - number of elements

  @return The function does not return anything.

  Created by Tomas Koudelka, 13.3.2002
*/
void mechtop::alloc_prep (long nn,long ne)
{
  nodes = new node [nn];
  elements = new element [ne];
}



/**
  Function allocates array for meaning of DOFs.
   
  @return The function does not return anything.

  Created by JK, 1.2.2005
*/
void mechtop::alloc_meaning ()
{
  long i;
  
  for (i=0;i<nn;i++){
    nodes[i].alloc_meaning (i);
  }
}



/**
  Function allocates all necessary arrays for problems with changing
  number of nodes, elements and DOFs.
   
  @return The function does not return anything.

  Created by JK, 7.11.2006
*/
void mechtop::alloc_growstr ()
{
  long i;
  
  for (i=0;i<nn;i++){
    nodes[i].alloc_growstr (i);
  }
  for (i=0;i<ne;i++){
    elements[i].alloc_growstr (i);
  }
  
}



/**
  Function defines meaning of DOFs.
   
  @return The function does not return anything.

  Created by JK, 1.2.2005
*/
void mechtop::define_meaning ()
{
  long i;
  elemtype te;
  
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      te = give_elem_type (i);
      
      switch (te){
      case beam2d:{
	Beam2d->define_meaning (i);
	break;
      }
      case beam3d:{
	Beam3d->define_meaning (i);
	break;
      }
      case planeelementlq:{
	Pelq->define_meaning (i);
	break; 
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
}



/**
  Function writes elements to the opened text file out.

  @param out, pointer to the opened text file
   
  @return The function does not return anything.

  Created by JK, 1.2.2005
*/
void mechtop::write_elements(FILE *out)
{
  long i,j;
  
  fprintf(out, "\nELEMENTS\n");
  for (i = 0; i < ne; i++){
    if (Gtm->leso[i]==1){
      fprintf(out, "%ld", i);
      switch (elements[i].te){
      case planeelementlt:
	fprintf(out, " %d", TriangleLinear);
	for (j = 0; j < Gtm->gelements[i].nne; j++)
	  fprintf(out, " %ld", Gtm->gelements[i].nodes[j]);
	break;
      case planeelementqt:
        fprintf(out, " %d", TriangleQuadratic);
        for (j = 0; j < Gtm->gelements[i].nne; j++)
          fprintf(out, " %ld", Gtm->gelements[i].nodes[j]);
        break;
      case planeelementrotlt:
        fprintf(out, " %d", TriangleLinear);
        for (j = 0; j < Gtm->gelements[i].nne; j++)
          fprintf(out, " %ld", Gtm->gelements[i].nodes[j]);
        break;
      case planeelementlq:
        fprintf(out, " %d", ISOLinear2D);
        for (j = 0; j < Gtm->gelements[i].nne; j++)
          fprintf(out, " %ld", Gtm->gelements[i].nodes[j]);
        break;
      case planeelementqq:
        fprintf(out, " %d", ISOQuadratic2D);
        for (j = 0; j < Gtm->gelements[i].nne; j++)
          fprintf(out, " %ld", Gtm->gelements[i].nodes[j]);
        break;
      case planeelementrotlq:
        fprintf(out, " %d", ISOLinear2D);
        for (j = 0; j < Gtm->gelements[i].nne; j++)
          fprintf(out, " %ld", Gtm->gelements[i].nodes[j]);
        break;
      default:
        print_err("unsupported element type is required", __FILE__, __LINE__, __func__);
      }
      fprintf(out, "\n");
    }
  }
}



/**
  Function generates code numbers of Lagrange multipliers.
   
  @param n - on input: number of DOFs without multipliers
             on output: number of DOFs with multipliers
	      
  @return The function returns number of DOFs with multipliers in the parameter n.

  Created by JK, 7.12.2002
*/
void mechtop::gencodnumlagrmult (long &n)
{
  long i,j,ndofn,nl,nmult,*cnl,*cnu;
  
  n++;
  
  for (i=0;i<nln;i++){
    nl=Gtm->lgnodes[i].nl;
    ndofn=Gtm->gnodes[Gtm->lgnodes[i].nodes[0]].ndofn;
    Gtm->lgnodes[i].ndofn=ndofn;
    Gtm->lgnodes[i].cn = new long* [nl+1];
    
    if (Mp->tlm==1){
      nmult=2;
      Gtm->lgnodes[i].nmult = nmult;
      cnl = new long [ndofn];
      cnu = new long [ndofn];
      
      Gtm->lgnodes[i].cn[0] = new long [nmult];
      Gtm->lgnodes[i].cn[0][0]=0;
      Gtm->lgnodes[i].cn[0][1]=0;
      
      for (j=1;j<nl;j++){
	give_node_code_numbers (Gtm->lgnodes[i].nodes[j-1],cnl);
	give_node_code_numbers (Gtm->lgnodes[i].nodes[j],cnu);
	
	Gtm->lgnodes[i].cn[j] = new long [nmult];
	if (cnl[0]>0 || cnl[2]>0 || cnu[0]>0 || cnu[2]>0){
	  Gtm->lgnodes[i].cn[j][0]=n;  n++;
	}
	if (cnl[1]>0 || cnu[1]>0){
	  Gtm->lgnodes[i].cn[j][1]=n;  n++;
	}
	
      }
      Gtm->lgnodes[i].cn[nl] = new long [nmult];
      Gtm->lgnodes[i].cn[nl][0]=0;
      Gtm->lgnodes[i].cn[nl][1]=0;
      
      delete [] cnl;  delete [] cnu;
    }
    
    if (Mp->tlm==2){
      nmult=4;
      Gtm->lgnodes[i].nmult = nmult;
      cnl = new long [ndofn];
      cnu = new long [ndofn];
      
      Gtm->lgnodes[i].cn[0] = new long [nmult];
      Gtm->lgnodes[i].cn[0][0]=0;
      Gtm->lgnodes[i].cn[0][1]=0;
      Gtm->lgnodes[i].cn[0][2]=0;
      Gtm->lgnodes[i].cn[0][3]=0;
      
      for (j=1;j<nl;j++){
	give_node_code_numbers (Gtm->lgnodes[i].nodes[j-1],cnl);
	give_node_code_numbers (Gtm->lgnodes[i].nodes[j],cnu);
	
	Gtm->lgnodes[i].cn[j] = new long [nmult];
	if (cnl[0]>0 || cnl[4]>0 || cnu[0]>0 || cnu[4]>0){
	  Gtm->lgnodes[i].cn[j][0]=n;  n++;
	}
	if (cnl[1]>0 || cnl[3]>0 || cnu[1]>0 || cnu[3]>0){
	  Gtm->lgnodes[i].cn[j][1]=n;  n++;
	}
	if (cnl[2]>0 || cnu[2]>0){
	  Gtm->lgnodes[i].cn[j][2]=n;  n++;
	}
	if (cnl[5]>0 || cnu[5]>0){
	  Gtm->lgnodes[i].cn[j][3]=n;  n++;
	}
	
      }
      Gtm->lgnodes[i].cn[nl] = new long [nmult];
      Gtm->lgnodes[i].cn[nl][0]=0;
      Gtm->lgnodes[i].cn[nl][1]=0;
      Gtm->lgnodes[i].cn[nl][2]=0;
      Gtm->lgnodes[i].cn[nl][3]=0;
      
      delete [] cnl;  delete [] cnu;
    }
  }
  n--;
}



/**
  The function searches adjacenet elements.
  
  @return The function does not return anything.

  Created by JK, 
*/
/*
void mechtop::adjacelem (void)
{
  long i, j, k, l, m, ned, nned, nedc, nnedc, net;
  long *enod, *et, *enodc;

  adjelem = new long *[ne];
  memset(adjelem, 0, sizeof(*adjelem)*ne);
  
  for (i = 0; i < ne; i++){
    if (Gtm->leso[i]==1){
      ned  = give_ned(i);
      if (ned < 3)
        print_warning("element number %ld has number of edges < 3", __FILE__, __LINE__ ,__func__, i+1);

      Gtm->gelements[i].edgn = new long [ned];
      adjelem[i] = new long[ned];
      for (j = 0; j < ned; j++)
	{
	  adjelem[i][j] = -1;
	  Gtm->gelements[i].edgn[j] = -1;
	}
    }
  }
  tned = 0;
  for (i = 0; i < ne; i++){
    if (Gtm->leso[i]==1){
      ned  = give_ned(i);
      for (j = 0; j < ned; j++)
	{
	  if (Gtm->gelements[i].edgn[j] > -1)
	    continue;
	  nned = give_nned(i);
	  enod = new long [nned];
	  memset(enod, 0, sizeof(*enod)*nned);
	  give_edge_nodes (i, j, enod);
	  net = 0;
	  for (k = 0; k < nned; k++)
	    net += Gtm->nadjelnod[enod[k]];
	  et = new long [net];
	  memset(et, 0, sizeof(*et)*net);
	  l = 0;
	  for (k = 0; k < nned; k++)
	    {
	      for (m = 0; m < Gtm->nadjelnod[enod[k]]; m++)
		{
		  et[l] = Gtm->adjelnod[enod[k]][m];
		  l++;
		}
	    }
	  for (k = 0; k < net; k++)
	    {
	      if (et[k] == i)
		continue;
	      nedc = give_ned(et[k]);
	      for (m = 0; m < nedc; m++)
		{
		  if (Gtm->gelements[et[k]].edgn[m] > -1)
		    continue;
		  nnedc = give_nned(et[k]);
		  enodc = new long [nedc];
		  memset(enodc, 0, sizeof(*enodc)*nnedc);
		  give_edge_nodes (et[k], m, enodc);
		  if (compare_edges(enod, enodc, nned))
		    {
		      adjelem[i][j]     = et[k];
		      adjelem[et[k]][m] = i;
		      Gtm->gelements[i].edgn[j]     = tned;
		      Gtm->gelements[et[k]].edgn[m] = tned;
		      delete [] enodc;
		      break;
		    }
		  delete [] enodc;
		}
	      if (adjelem[i][j] > -1)
		break;
	    }
	  delete [] et;
	  delete [] enod;
	  Gtm->gelements[i].edgn[j] = tned;
	  tned++;
	}
    }
  }
}
*/



/**
  The function compares nodal numbers of two edges.

  @param enod - array with node numbers of the first compared edge
  @param enodc - array with node numbers of the second compared edge
  @param nned - number of components of enod and enodc (i.e. number of edge nodes)

  @retval 0 - edges are different
  @retval 1 - edges are identical

  Created by JK,
*/
long mechtop::compare_edges (long *enod, long *enodc, long nned)
{
  long i, j, nsn = 0;

  for (i = 0; i < nned; i++)
  {
    for (j = 0; j < nned; j++)
    {
      if (enod[i] == enodc[j])
      {
        nsn++;
        break;
      }
    }
  }
  if (nsn == nned)
    return 1;

  return 0;
}



/*
void mechtop::eadjacelem (void)
{
  long i, j, ned, ided;

  eadjelem = new long *[tned];
  memset (eadjelem, 0, sizeof(*eadjelem)*tned);
  for (i = 0; i < tned; i++)
  {
    eadjelem[i] = new long[2];
    eadjelem[i][0] = -1;
    eadjelem[i][1] = -1;
  }
  
  for (i = 0; i < ne; i++){
    if (Gtm->leso[i]==1){
      ned  = give_ned(i);
      for (j = 0; j < ned; j++)
	{
	  ided = Gtm->gelements[i].edgn[j];
	  if (eadjelem[ided][0] < 0)
	    {
	      eadjelem[ided][0] = i;
	      continue;
	    }
	  if (eadjelem[ided][1] < 0)
	    {
	      eadjelem[ided][1] = i;
	      continue;
	    }
          print_err("one edge connects more than 2 elements,\n"
                    " the function can be applied only on 2D problems", __FILE__, __LINE__, __func__);
	}
    }
  }
}
*/



/**
  Function returns nodal displacements.
  Function is intended for molecular computations.
   
  Created by JK, 19.6.2005
*/
void mechtop::nodedisplacements ()
{
  long i,j,ndofn,lcid;
  double *r;
  
  //  the same number of unknowns at nodes is assumed at this version
  //  in case of different numbers of unknowns, function must be modified
  ndofn = give_ndofn (0);
  
  //  only one loading case is assumed at this version
  //  it must be generalized for more loading cases
  lcid=0;
  
  //  possible allocation
  if (nodedispl==NULL){
    nodedispl = new double* [nn];
    for (i=0;i<nn;i++){
      ndofn = give_ndofn (i);
      nodedispl[i] = new double [ndofn];
      for (j=0;j<ndofn;j++){
	nodedispl[i][j]=0.0;
      }
    }
  }
  
  //  loop over nodes
  for (i=0;i<nn;i++){
    ndofn = give_ndofn (i);
    r = new double [ndofn];
    //  nodal displacements
    noddispl (lcid,r,i);
    for (j=0;j<ndofn;j++){
      nodedispl[i][j]=r[j];
    }
    delete [] r;
  }
}



/**
  The function returns the first component of nodal displacements for the 
  selected nodes given by parameter nodes.

  @param nodes - %vector of selected nodes
  @param u - %vector of displacements (output)

  @return The function returns the first displacement component of selected nodes in 
          the parameter u.

  Created by JK, 23.6.2005
*/
void mechtop::give_noddispl_1d (ivector &nodes,vector &u)
{
  long i;
  
  for (i=0;i<nodes.n;i++){
    u[i]=nodedispl[nodes[i]][0];
  }
  
}



/**
  The function returns the first two components of nodal displacements for the 
  selected nodes given by parameter nodes.

  @param nodes - %vector of selected nodes
  @param u - %vector of the first displacement component (output)
  @param v - %vector of the second displacement component (output)

  @return The function returns displacement components at selected nodes in 
          parameters u and v.

  Created by JK, 23.6.2005
*/
void mechtop::give_noddispl_2d (ivector &nodes,vector &u,vector &v)
{
  long i;
  
  for (i=0;i<nodes.n;i++){
    u[i]=nodedispl[nodes[i]][0];
    v[i]=nodedispl[nodes[i]][1];
  }
}



/**
  The function returns first three components of nodal displacements for the 
  selected nodes given by parameter nodes.

  @param nodes - %vector of selected nodes
  @param u - %vector of the first displacement component (output)
  @param v - %vector of the second displacement component (output)
  @param w - %vector of the third displacement component (output)

  @return The function returns displacement components at selected nodes in 
          parameters u, v and w.

  Created by JK, 23.6.2005
*/
void mechtop::give_noddispl_3d (ivector &nodes,vector &u,vector &v,vector &w)
{
  long i;
  
  for (i=0;i<nodes.n;i++){
    u[i]=nodedispl[nodes[i]][0];
    v[i]=nodedispl[nodes[i]][1];
    w[i]=nodedispl[nodes[i]][2];
  }
  
}



/**
  Function initializes mechtop class from the initialized siftop structure.
  The function is used in the preprocessors esspecially.

  @param top - initialized siftop structure

  @return The function does not return anything.
 
  Created by Tomas Koudelka
*/
void mechtop::init_from_siftop (siftop *top)
{
  long i,j;

  //  node data
  nn = top->nn;
  //  element data
  ne = top->ne;
  // export and allocation of general topology
  top->exporttop(Gtm);

  if (Mespr==1)  fprintf (stdout,"\n number of nodes  %ld",nn);
  if (Mespr==1)  fprintf (stdout,"\n number of elements  %ld",ne);
  nodes = new node [nn];
  elements = new element [ne];

  for (i=0; i < ne; i++){
    if (Gtm->leso[i]==1){
      for(j=0; j<Gtm->gelements[i].give_nne(); j++)
	Gtm->gelements[i].nodes[j]--;
    }
  }
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      elements[i].tm = new mattype[1];
      elements[i].idm = new long[1];
      elements[i].idm[0] = top->elements[i].prop-1;
      switch (top->elements[i].type)
	{
	case ISOLinear1D:
	  elements[i].te = bar2d;
	  if (Bar2d == NULL)
	    Bar2d = new barel2d;
	  break;
	case ISOQuadratic1D:
	  elements[i].te = barq2d;
	  if (Barq2d == NULL)
	    Barq2d = new barelq2d;
	  break;
	case TriangleLinear:
	  elements[i].te = planeelementlt;
	  if (Pelt == NULL)
	    Pelt = new planeelemlt;
	  break;
	case TriangleQuadratic:
	  elements[i].te =planeelementqt;
	  if (Peqt == NULL)
	    Peqt = new planeelemqt;
	  break;
	case ISOLinear2D:
	  elements[i].te = planeelementlq;
	  if (Pelq == NULL)
	    Pelq = new planeelemlq;
	  break;
	case ISOQuadratic2D:
	  elements[i].te = planeelementqq;
	  if (Peqq == NULL)
	    Peqq = new planeelemqq;
	  break;
	case TetrahedronLinear:
	  elements[i].te = lineartet;
	  if (Ltet == NULL)
	    Ltet = new lintet;
	  break;
	case TetrahedronQuadratic:
	  elements[i].te = quadrtet;
	  if (Qtet == NULL)
	    Qtet = new quadtet;
	  break;
	case VedgeLinear:
	  elements[i].te = linearwed;
	  if (Lwed == NULL)
	    Lwed = new linwedge;
	  break;
	case VedgeQuadratic:
	  elements[i].te = quadrwed;
	  if (Qwed == NULL)
	    Qwed = new quadwedge;
	  break;
	case ISOLinear3D:
	  elements[i].te = linearhex;
	  if (Lhex == NULL)
	    Lhex = new linhex;
	  break;
	case ISOQuadratic3D:
	  elements[i].te = quadrhex;
	  if (Qhex == NULL)
	    Qhex = new quadhex;
	  break;
	default:
	  print_err("unknown type of element is used", __FILE__, __LINE__, __func__);
	  abort();
	}
    }
  }
}



/**
  Function cleans strains, stresses and other quantities defined at nodes.
  The function is used especially in stochastic or fuzzy computations.
   
  @return The function does not return anything.
 
  Created by JK, 23.8.2005
*/
void mechtop::clean_nodes ()
{
  long i;
  
  for (i=0;i<nn;i++){
    nodes[i].clean (Mb->nlc);
  }
}



/**
  Function establishes vector of initial displacements.
   
  @return The function does not return anything.
 
  Created by JK, 28.2.2006
*/
void mechtop::initial_displ ()
{
  long i,j,k,ndofe,lcid,*cn;
  double *r;
  
  //  load case id must be equal to zero
  lcid=0;
  
  for (i=0;i<ne;i++){
    
    j=Gtm->leso[i];
    k=Gtm->gelements[i].auxinf;
    
    if (j==1 && k==0){
      //  added element
      //  number of DOFs on element
      ndofe = give_ndofe (i);
      
      r = new double [ndofe];
      cn = new long [ndofe];
      
      //  code numbers
      give_code_numbers (i,cn);
      //  nodal displacements
      eldispl (lcid,i,r,cn,ndofe);
      //  definition of initial displacements
      elements[i].initdisplacement (r,ndofe);
      
      Gtm->gelements[i].auxinf=1;
      
      delete [] cn;
      delete [] r;
    }
  }
}



/**
  Function saves nodal unknowns from the array lhs to nodes.
  The function is used in problems with growing number of nodes and elements.
   
  @return The function does not return anything.
 
  Created by JK, 2.6.2006
*/
void mechtop::lhs_save (double *lhs,long id)
{
  long i,j,ndofn;
  long *cn;
  
  if (id==1){
    for (i=0;i<nn;i++){
      ndofn = give_ndofn (i);
      cn = new long [ndofn];
      give_node_code_numbers (i,cn);
      
      for (j=0;j<ndofn;j++){
	//nodes[i].nodval[j]=0.0;
	if (cn[j]>0){
	  nodes[i].nodval[j]=lhs[cn[j]-1];
	}
      }
      delete [] cn;
    }
  }
  
  if (id==2){
    for (i=0;i<nn;i++){
      ndofn = give_ndofn (i);
      cn = new long [ndofn];
      give_node_code_numbers (i,cn);
      
      for (j=0;j<ndofn;j++){
	//nodes[i].nvgraph[j]=0.0;
	if (cn[j]>0){
	  nodes[i].nvgraph[j]=lhs[cn[j]-1];
	}
      }
      delete [] cn;
    }
  }
}



/**
  Function restores nodal unknowns from nodes to the array lhs.
  The function is used in problems with growing number of nodes and elements.

  @param lhs - array of left hand side i.e. array of unknowns (output)
  @param n - total number of DOFs (dimension of array lhs)
   
  @return The function restores saved unknowns at nodes to the %vector given by 
          parameter lhs.

  Created by JK, 2.6.2006
*/
void mechtop::lhs_restore (double *lhs,long n)
{
  long i,j,ndofn;
  long *cn;
  
  for (i=0;i<n;i++){
    lhs[i]=0.0;
  }

  for (i=0;i<nn;i++){
    ndofn = give_ndofn (i);
    cn = new long [ndofn];
    give_node_code_numbers (i,cn);
    
    for (j=0;j<ndofn;j++){
      if (cn[j]>0){
	lhs[cn[j]-1]=nodes[i].nodval[j];
      }
    }
    delete [] cn;
  }
}



/**
  The function saves righ hand side %vector (nodal forces) to
  nodes. 

  @param rhs - right hand side %vector

  @return The function does not return anything.

  Created by JK,
*/
void mechtop::rhs_save (double *rhs)
{
  long i,j,ndofn;
  long *cn;
  
  for (i=0;i<nn;i++){
    
    if (Gtm->nm[i]==1){
      ndofn = give_ndofn (i);
      cn = new long [ndofn];
      give_node_code_numbers (i,cn);
      
      for (j=0;j<ndofn;j++){
	nodes[i].nvforce[j]=0.0;
	if (cn[j]>0){
	  nodes[i].nvforce[j]=rhs[cn[j]-1];
	}
      }
      delete [] cn;
    }
  }
}



/**
  The function restores righ hand side %vector (nodal forces) from 
  the previously saved nodal values. 

  @param rhs - right hand side %vector (output)
  @param n - number of rhs components

  @return The function returns restored right hand side %vector in the parameter rhs. 

  Created by JK,
*/
void mechtop::rhs_restore (double *rhs,long n)
{
  long i,j,ndofn;
  long *cn;

  for (i=0;i<n;i++){
    rhs[i]=0.0;
  }
  for (i=0;i<nn;i++){
    if (Gtm->nm[i]==1){
      ndofn = give_ndofn (i);
      cn = new long [ndofn];
      give_node_code_numbers (i,cn);
      
      for (j=0;j<ndofn;j++){
	if (cn[j]>0){
	  rhs[cn[j]-1]+=nodes[i].nvforce[j];
	}
      }
      delete [] cn;
    }
  }
}



/**
  The function checks correct assignment of time functions of particular nodes 
  comparing to assignment of element time functions. It is used for growing structures especially.
  
  @return The function does not return anything.

  Created by TKr,
*/
long mechtop::mesh_check(void)
{
  long i,j,err=0,kk,jj;

  //elems and nodes check
  for(i=0;i<Mt->ne;i++){
    for(j=0;j<Gtm->gelements[i].nne;j++){
      
      jj = Gtm->gelements[i].nodes[j];
      for(kk = 0; kk < Gtm->gnodes[jj].ndofn;kk++){
	
	if((Gtm->gelements[i].tgf) < (Gtm->gnodes[jj].tgf[kk])){
	  print_err("different element node(%ld) time function number=%ld than element(%ld) time function number=%ld\n"
                    " (node number=%ld, element number=%ld)",
                    __FILE__, __LINE__, __func__, j+1,Gtm->gnodes[jj].tgf[kk]+1,i+1,Gtm->gelements[i].tgf+1,Gtm->gelements[i].nodes[j]+1,i+1);
	  err = 1;
	  return err;
	}
      }
    }
  }
  return err;
}
