#ifndef SCALDAMTIME_H
#define SCALDAMTIME_H

#include "iotools.h"
#include "alias.h"
#include "strretalg.h"
struct matrix;
struct vector;
struct atsel;

/**
  This class defines scalar isotropic damage material model for time dependent problems.
  The different type of norms for the computing parameters of the damage function can be used.
  New order of internal variables in the other array :
  ------------------------------------------------------------------------------------
  0               : maximum of previous equivalent strain
  1               : actual damage
  ------------------------------------------------------------------------------------
  Total number of components of other array : 2


  Old order of internal variables in the other array fro alternative computations of damge from increments:
  ------------------------------------------------------------------------------------
  0 .. ncompstr-1          : previous total strains
  ncompstr .. 2*ncompstr-1 : increment of total strains
  2*ncompstr               : previous equivalent strain
  2*ncompstr+1             : actual damage
  2*ncompstr+2             : previous value of Young modulus
  2*ncompstr+3 .. 3*ncompstr+2 : previous value of stresses
  3*ncompstr+3 .. 4*ncompstr+2 : stresses increment
  ------------------------------------------------------------------------------------
  Total number of components of other array : 4*ncompstr+3

*/
class scaldamtime
{
 public:
  scaldamtime (void);
  ~scaldamtime (void);
  void read (XFILE *in);
  void damfuncpar(long ipp, vector &eps, vector &kappa);
  void der_damfuncpar(long ipp, vector &eps, vector &kappa, vector &depseq);
  double damfunction(long ipp, vector &kappa);
  double damage_rate(long ipp, vector &kappao, vector &dkappa, double omegao, double eo, double e);
  void matstiff (matrix &d,long ipp,long ido);
  void elmatstiff (matrix &d,long ipp);
  void nlstresses (long ipp, long im, long ido);
  double epsefunction (long ipp);
  void changeparam (atsel &atm,vector &val);
  double givedamage(long ipp, long ido);
  double give_actual_ft (long ipp, long im, long ido);
  void initvalues (long ipp, long im, long ido);
  void updateval (long ipp, long im, long ido);

  /// max effective strain at peak
  double indam;
  /// tensile strength
  double st;
  /// determines the softening -> corresponds to crack opening (not strain) when tension stress vanishes
  double uf;
  /// parameter for energy norm
  double c;
  /// parameter for von Mises norm - ratio of compression and tensile strength
  double kf;
  /// type of function for evaluation damage funtion parameter
  paramf_type ftype;
  /// correction of disipated energy switch
  corr_disip_en cde;
  ///  stress return algorithm
  strretalg sra;
};

#endif
