#include "strretalg.h"
#include "alias.h"
#include "global.h"



/**
  Constructor initializes data members to zero or default values.

  Created by JK,
*/
strretalg::strretalg ()
{
  zero=0.0;
  ni=0;
  err=0.0;
}



/**
  The destructor is defined only for the formal purposes.
  
  Created by JK,
*/
strretalg::~strretalg ()
{
}



/**
  The function reads parameters for stress-return algorithm from the opened text file given
  by the parameter in.

  @param in - pointer to the opened input text file
  
  @return The function does not return anything.

  Created by JK,
*/
void strretalg::read (XFILE *in)
{
  //  type of stress return algorithm
  xfscanf (in,"%m", &stressretalgtype_kwdset,(int*)&tsra);
  
  switch (tsra){
  case nostressretalg:{
    if (Mespr==1)
      fprintf (stdout,"\n\n no stress return algorithm is required (file %s, line %d).\n",__FILE__,__LINE__);
    break;
  }
  case cp:{
    xfscanf (in,"%ld %lf",&ni,&err);
    break;
  }
  case gsra:{
    xfscanf (in,"%ld %lf",&ni,&err);
    break;
  }
  default:
    print_err("unknown stress return algorithm is required", __FILE__, __LINE__, __func__);
  }
  
}


/**
  The function returns required type of stress-return algorithm.

  @return The function returns required type of stress-return algorithm.

  Created by JK,
*/
stressretalgtype strretalg::give_tsra ()
{
  return tsra;
}



/**
  The function returns required maximum number of iterations of reuired stress-return algorithm.

  @return The function returns maximum number of iterations.

  Created by JK,
*/
long strretalg::give_ni ()
{
  return ni;
}



/**
  The function returns required error of stress-return algorithm.

  @return The function returns required error of stress-return algorithm.

  Created by JK,
*/
double strretalg::give_err ()
{
  return err;
}
